function WREGv1_05_agvPRL()
% v1: KEng and YChen original code
%
% v1.01:
% JEK 4.16.10: change so that regression coefficients are output to 3
%             decimal places (instead of 2 used in v1.)
% v1.02:
% JEK 11.8.2010: will change so that takes in daily time series as either
%               a single file (if it exists), or as before, in separate
%               files for each station.
% v1.03:
% JEK 6.9.2011: added invXLX output for user-defined WLS (still need to
% test more carefully - so the output line is commented out for now, pending
%   additional testing)
% JEK 6.9.2011: added MSE to outputs for GLS and WLS, changed output format
% slightly - units shown next to each metric rather than as asterisked note
% AGV 12.7.2011: corrected Pseudo R2 for regional regression output for GLS
% and WLS
%
% v1.04:
% JEK 2.9.2012: reactivated invXLX output for userWLS after checking
%  this invXLX output should match Tasker WLS for same weights.
%  that is, it includes MSE 
% 
% v1.05:
% JEK 3.20.2013: added AVP and Model error variance in log units to conventional outputs, as
% applicable
% JEK 3.29.2013: added VP for individual streamgages to conventional
% outputs

versionnum = '1.05';
versiondate = 'last modified August 8, 2015';

%%%%%%%%%%%%
%Input Data%
%%%%%%%%%%%%
%IC  Record entries in Command Window to EventLog.txt
delete('EventLog.txt');
diary('EventLog.txt');

%IC  load UserWLS.txt;     %Users weighting matrix
FlowChar=importdata('FlowChar.txt', '\t', 1);   %tab delimited text file '\t' and specifying header on row 1
LP3K=importdata('LP3K.txt', '\t', 1);
LP3s=importdata('LP3s.txt', '\t', 1);
LP3G=importdata('LP3G.txt', '\t', 1);
SiteInfo=importdata('SiteInfo.txt', '\t', 1);
size(SiteInfo.data,2)
%-----------------------------------------
%added by agv 8/6/15 for PRL fix
%LP3RLS contains pseudo record lengths for each station for each AEP, matches format of LP3G file.
LP3PRL=importdata('LP3PRL.txt', '\t', 1);
%-----------------------------------------

%IC  find total number of sites and records
global NumSites NumRows
NumSites=size(SiteInfo.data,1)
NumRows=sum(SiteInfo.data(:,4));


%IC  Set IGUI=1 to enable GUI
%IC  Set IGUI=999 to disable GUI (When Input GUI is disabled, remember to manually set input)
IGUI_LoadStatus=999; %IC  This part has been moved to GUI_Model
IGUI_Choose=1;
IGUI_Transform=1;
IGUI_Model=1;
IGUI_Summary=1;

if IGUI_LoadStatus==1  %IC (Loading Status) GUI enabled
%IC  Disable warning message for case-sensitive match
    warning('off','MATLAB:dispatcher:InexactMatch')
    
%IC  Create "Loading Status" figure
    h_LSfig=figure('Name','WREG ver 1.0 - Loading Status',...
        'NumberTitle','off',...
        'MenuBar','none',...
        'Color',[0.83 0.81 0.78],...
        'Units','normalized',...
        'Position',[0.288 0.356 0.424 0.288],...
        'Resize','on');

    h_StatusText=uicontrol(h_LSfig,...
        'Units','normalized',...
        'Position',[0.011 0.53 0.98 0.226],...
        'Style','text',...
        'String','Loading input files...',...
        'FontUnits','normalized',...
        'FontSize',0.641,...
        'FontName','MS Sans Serif',...
        'HorizontalAlignment','center',...
        'BackgroundColor',[0.83 0.81 0.78]);
    
    h_StatusOK=uicontrol(h_LSfig,...
        'Units','normalized',...
        'Position',[0.394 0.126 0.193 0.222],...
        'Style','pushbutton',...
        'String','OK',...
        'FontUnits','normalized',...
        'FontSize',0.654,...
        'FontName','MS Sans Serif',...
        'HorizontalAlignment','center',...
        'BackgroundColor',[0.83 0.81 0.78],...
        'Enable','off');

%IC put in USGS logo (original size 600*229 pixels)
    h_USGSAxes=axes('Units','normalized',...
        'Position',[0.836 0.026 0.147 0.135]);
    uistack(h_USGSAxes,'bottom');
    USGSlogo=imread('USGSlogo.JPG');
    h_USGSImage=imagesc(USGSlogo);
    colormap gray
%IC    set(h_USGSImage,'Alphadata',.5);
    set(h_USGSAxes,'Handlevisibility','off',...
        'Visible','off');

%IC Adjust screen size (developed in 1280*800)
    old_u_units=get(gcf,'Units');
    upos=get(gcf,'Position');
    fig_ratio=(upos(4)*800)/(upos(3)*1280);
    set(gcf,'Units','pixels');
    upos=get(gcf,'Position');
    upos_new=upos;
    upos_new(4)=upos_new(3)*fig_ratio;
    set(gcf,'Position',[upos_new(1) upos_new(2) upos_new(3) upos_new(4)]);
    set(gcf,'Units',old_u_units);
    upos=get(gcf,'Position');
    upos(2)=(1-upos(4))/2;
    set(gcf,'Position',[upos(1) upos(2) upos(3) upos(4)]);


% JEK 9.13.2010 for v1.02: read in a single file to set up the correlation
% matrix for annual flow series.  This is done to avoid problems with
% windows formatting.  File is created with WREG shell program.
% Commands for reading in individual annual time series files commented
% out.
% JEK 11.15.10: Thought this was handled here, but apparently it is
% actually handled in GUI_Model.m function LoadATS().  Additional code
% added there for error checking and proper naming of vars.
if(exist('USGSAnnualTimeSeries.txt', 'file')==2)
    disp('Using file: USGSAnnualTimeSeries.txt')
    CorrelationInput=importdata('USGSAnnualTimeSeries.txt', '\t', 1)
    
%If the premade input file doesn't exist,
%the "CorrelationInput" file will be assembled by reading in separate .txt
%files for each site from the same directory that contain "USGS" in the
%filename.
else
   AnnualFlows=dir('*USGS*.txt')
   disp('Using files: *USGS*.txt')
   ajkm_done_col=0;
   ajkm_row=0;
   for ajkm=1:length(AnnualFlows)
      filename=AnnualFlows(ajkm).name;
      data1=importdata(filename);
      ajkm_row=ajkm_row+size(data1.textdata,1);
      if size(data1.data,2)==2
        if ajkm==1
            CorrelationInput=data1.data;
        else
            CorrelationInput=[CorrelationInput; data1.data];
        end
        ajkm_done_col=ajkm_done_col+1;
      else
        set(h_StatusText,'String','Error on loading input files.');
        set(h_StatusOK,'Enable','on','Callback','quit');
      end
   end
end

%if (ajkm_done_col==length(AnnualFlows))&(size(CorrelationInput,1)==ajkm_row)
%    set(h_StatusText,'String','Input files loaded.');
%    set(h_StatusOK,'Enable','on','Callback','close');
%else
%    set(h_StatusText,'String','Error on loading input files.');
%    set(h_StatusOK,'Enable','on','Callback','quit');
%end
%uiwait(gcf);

end  %IC end (Loading Status) GUI


%if IGUI_Model==1
    global C LFCORR2 SIZELF2
%end

%if IGUI_Summary==1
    global B_Transform Name_B_Selected B_Constant COEFFINFO1
    global N_A_Selected A_Transform Name_A_Selected A_Constant COEFFINFO2
%end

A1=SiteInfo.data(:, 9:size(SiteInfo.data,2));       %Predictors

B1=FlowChar.data(:, 2:size(FlowChar.data,2));       %Predictands
C=SiteInfo.data(:, 2:3);                            %Lat and Long
%E=[SiteInfo.data(:, 5) LP3K.data(:, 4) SiteInfo.data(:, 6)];    %Lsigma, K, & g values
%IC  LFCORR2=CorrelationInput(:, 1:2);              %Annual Series Values
SIZELF2=SiteInfo.data(:, 4);                        %No. of Annual Series values
RegionSkew=0.302;                                   %MSE of regional skew. The default value is 0.302 from Bulletin 17B
%IC  UW=UserWLS;                                         %User defined weighting matrix

%if IGUI_Choose==1  %IC (Choosing) GUI enabled
    global A_Selected B_Selected N_A_Selected Name_A_Selected Name_B_Selected
    
%IC  Set up parameters for listbox
    N_list1=size(FlowChar.data,2)-1;  %IC number of options in 1st listbox
    N_list2=size(SiteInfo.data,2)-8;  %IC number of options in 2nd listbpx
%IC  Creat option name strings for use in listbox
%IC  Option names stored in FlowChar.colheaders and SiteInfo.colheaders
    for iNlist=1:N_list1
        if iNlist==1
            Name_list1=FlowChar.colheaders(iNlist+1);
        else
            Name_list1=strcat(Name_list1,'|',FlowChar.colheaders(iNlist+1));
        end
    end
    for iNlist=1:N_list2
        if iNlist==1
            Name_list2=SiteInfo.colheaders(iNlist+8);
        else
            Name_list2=strcat(Name_list2,'|',SiteInfo.colheaders(iNlist+8));
        end
    end
    Name_list1=char(Name_list1);
    Name_list2=char(Name_list2);
    
    GUI_Choose(Name_list1,Name_list2);
    
%IC  B_Selected is the column number selected in matrix B1
%IC  A_Selected is an array containing the column numbers 
%IC     selected in matrix A1
%IC     A_Selected=[2 3] means column 2 and 3 are selected
    
    for ingcd=1:size(A_Selected,2)
        for ingc=1:size(A1,2)
            if ingc==A_Selected(1,ingcd)
                A2(:,ingcd)=A1(:,ingc); %User selected input matrix for independent variables
            else
    
            end
        end
    end

%end  %IC end (Choosing) GUI

%Transformations to Predictors and Predictand Chosen by User
%if IGUI_Transform==1  %IC (Transformation) GUI enabled
    global A_Transform A_Constant B_Transform B_Constant
    
    B_Transform=0;
    B_Constant=0;
    A_Transform=0;
    A_Constant=0;
    
    GUI_Transform(N_A_Selected,Name_A_Selected,Name_B_Selected,A2,B1,B_Selected);
    
%IC  B_Transform is a number indicating the method of transformation
%IC  B_Transform=1 : none
%IC  B_Transform=2 : log10
%IC  B_Transform=3 : ln
%IC  B_Transform=4 : exp
%IC  B_Constant is a 1*4 matrix indicating the formula constants
%IC  B_Constant=[C1 C2 C3 C4]
%IC  A_Transform is a 1*(up to 5) matrix indicating the method 
%IC              of transformation
%IC  A_Constant is a (up to 5)*4 matrix indicating the formula constants

%end  %IC end (Transformation) GUI

%Predictor transform
for ingce=1:size(A_Constant,1)
    for ingcf=1:size(A2,1)
        if A_Transform(1,ingce)==2
            A(ingcf,ingce)=log10(((A_Constant(ingce,1)*A2(ingcf,ingce)^(A_Constant(ingce,2)))+A_Constant(ingce,3))^(A_Constant(ingce,4)));
        elseif A_Transform(1,ingce)==3
            A(ingcf,ingce)=log(((A_Constant(ingce,1)*A2(ingcf,ingce)^(A_Constant(ingce,2)))+A_Constant(ingce,3))^(A_Constant(ingce,4)));
        elseif A_Transform(1,ingce)==4
            A(ingcf,ingce)=exp(((A_Constant(ingce,1)*A2(ingcf,ingce)^(A_Constant(ingce,2)))+A_Constant(ingce,3))^(A_Constant(ingce,4)));
        else    %No Transformation A_Transform(1,ingce)==1
            A(ingcf,ingce)=(((A_Constant(ingce,1)*A2(ingcf,ingce)^(A_Constant(ingce,2)))+A_Constant(ingce,3))^(A_Constant(ingce,4)));
        end 
    end
end
%Predictand transform
for ingcg=1:size(B1,1)
    if B_Transform==2
        B1(ingcg,B_Selected)=log10(((B_Constant(1,1)*B1(ingcg,B_Selected)^(B_Constant(1,2)))+B_Constant(1,3))^(B_Constant(1,4)));
    elseif B_Transform==3
        B1(ingcg,B_Selected)=log(((B_Constant(1,1)*B1(ingcg,B_Selected)^(B_Constant(1,2)))+B_Constant(1,3))^(B_Constant(1,4)));
    elseif B_Transform==4
        B1(ingcg,B_Selected)=exp(((B_Constant(1,1)*B1(ingcg,B_Selected)^(B_Constant(1,2)))+B_Constant(1,3))^(B_Constant(1,4)));
    else    %No Transformation B_Transform==1
        B1(ingcg,B_Selected)=(((B_Constant(1,1)*B1(ingcg,B_Selected)^(B_Constant(1,2)))+B_Constant(1,3))^(B_Constant(1,4)));
    end 
end
B=B1;

ChY=B_Selected;    %Which column no. in the B matrix to use as predictand (ie YHat)
E=[LP3s.data(:, ChY+1) LP3K.data(:, ChY+1) LP3G.data(:, ChY+1)];    %Lsigma, K (chosen), & g values

%-----------------------------------------
%added by agv 8/6/15 for PRL fix
%LP3RLS contains pseudo record lengths for each station for each AEP, matches format of LP3G file.
PRL = [LP3PRL.data(:,ChY+1)] %Pseudo Record Length corresponding to chosen AEP
%------------------------------------------

%%%%%%%%%%%%
%User Input%
%%%%%%%%%%%%
%if IGUI_Summary==1
    global RegType ParFit UserWeight
    global COLSOrigRes2 OrigAVP1 MOrigResSq2
    global OLSR2 PR2 EYR SelGSQ2 ca2
%end

%Option 1 - Conventional multiple linear regression (=1) or region-of-influence regression (=2)
  RegType=1;
    %if =2 then the following info is required
      MaxR=10000000;      %Max distance in km to go from ungauged basin (=10000000 then it is PRoI if =1 then GRoI)
      nmin=8;            %Minimum number of stations to skim off the top of sorted stations by Euclidean distance to include in a regression (must be>5)
%Option 2 - 3 types of parameter fitting for regression models: OLS (=1), WLS (=2), and GLS (=3)
  ParFit=3;
    %if =2    
      UserWeight=999;   %=1 if using users own weighting matrix otherwise this is defaulted to a value =999
    % Default settings for distance-correlation plots used in GLS
    %if =3
      ConcYrsReqd=30;
    %Correlation function constants for plot and smoothed correlation values
      Theta1=0.98;
      Alpha1=0.03;
    
%%%%%%%%%%%%%%%%%
%NEW USER INPUTS%
%%%%%%%%%%%%%%%%%
%if ParFit=3
      UnSkew=999; %or else this is =999, this option lets people use the new method to account for the uncertainty in the skew values.    
      RT1=2; %10 year return period
      RT=(1/RT1); %Only used for program
      Exceed=1; %Exceedence(peaks-default=999) or Nonexceedence probability(low-flows=1)

%KE (4-21-08) If GLS is chosen for either RoI or conventional regression, the correlation smoothing function will need to be determined
%IC  if ParFit==3        %GLS parameter fitting
%IC      [dgeo,m,Conm]=CorrSmooth(C,LFCORR2,SIZELF2,Alpha1,Theta1,ConcYrsReqd);    %Calling subroutine to do correlation smoothing function
%IC      Alpha=Alpha1;   %This will equal whatever is chosen in GUI window
%IC      Theta=Theta1;   %This will equal whatever is chosen in GUI window
%IC  else
%IC      
%IC  end    
    

%if IGUI_Model==1  %IC (Model) GUI enabled
    global RegTypeGL MaxRGL nminGL ParFitGL UserWeightGL ConcYrsReqdGL AlphaGL ThetaGL
    global UWGL
    global dgeo m Conm
    global UnSkewGL RT1GL ExceedGL RegionSkewGL
    global flag_load
    
    flag_load=0;
    GUI_Model;

    %IC  Assign values from GUI to main program
    RegType=RegTypeGL
    MaxR=MaxRGL
    nmin=nminGL
    ParFit=ParFitGL
    UserWeight=UserWeightGL
    ConcYrsReqd=ConcYrsReqdGL
    Alpha=AlphaGL
    Theta=ThetaGL
    UW=UWGL
    UnSkew=UnSkewGL
    RT1=RT1GL
    Exceed=ExceedGL
    RegionSkew=RegionSkewGL

%end  %IC end (Model) GUI



%%% START OF MAIN CODE FOR SETTING UP AND RUNNING REGRESSION %%%%%%%%%%%%
%%% -------------------------------------------------------- %%%%%%%%%%%%
if RegType==1   %Conventional multiple linear regression
    GRNum=linspace(1,size(SiteInfo.data,1),size(SiteInfo.data,1))';
    GR=[SiteInfo.data(:,7) GRNum];  %Regional Skew Values at each gage & Site no
    if ParFit==3        %GLS parameter fitting
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %SIGMA REGRESSION FOR GLS REGRESSION%
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %GLOBAL Sigma Regression, so using global parameter values instead of localized RoI ones (Regressing by multiple linear regression between each stations std dev on annual peaks to corresponding basin attributes)
        GlobalPreds=[A(1:size(A,1), 1:size(A,2))];
        GXBasin=[ones(size(GlobalPreds,1),1) GlobalPreds(1:size(GlobalPreds,1), 1:size(A,2))];  %Predictors at each selected station
        GLSigma=[E(:,1)];                                                                       %Lsigma values
        SigCoeff=regress(GLSigma,GXBasin);
        
        ExtrS=[A(1:size(A,1),1:size(A,2)) B(1:size(B,1),1:size(B,2)) C(1:size(C,1),1:size(C,2)) E(1:size(E,1),1:size(E,2))];
        %The 'ExtrS' matrixs contains the following info: basin attributes, predictands, lat and long, at-site Lsigma, K, weighted skew, estimates sigma values for all the basins
        
        %Calculating the sample Std Dev, si, at each gauged basin in the RoI (Using si as estimator of sigma i)
        for ff=1:size(A,1)
            SumSigma=0;
            for fef=1:size(A,2)
                SumSigma=SumSigma+(SigCoeff(fef+1,1)*A(ff,fef));
            end
            Sigma(ff,1)=SigCoeff(1,1)+SumSigma;           
        end
                
        for ss=1:size(ExtrS,1)
            ExtrS(ss,size(A,2)+size(B,2)+size(C,2)+size(E,2)+2)=Sigma(ss,1);
        end
        
        % Call to solve GLS equations
        if UnSkew==1
            [FinalCoeff(:,1), SelGSQ, LambdaF]=NewLambdaMatrixFinalagvPRL(A,B,C,E,Sigma,m,dgeo,ExtrS,Theta,Alpha,ChY,Conm,RT,RegionSkew,Exceed,GR,PRL); 
        else
            [FinalCoeff(:,1), SelGSQ, LambdaF]=LambdaMatrixFinalagvPRL(A,B,C,E,Sigma,m,dgeo,ExtrS,Theta,Alpha,ChY,Conm,PRL);   %Calling out subroutine 'LambdaMatrix'
            %The output from this subroutine are the final coefficients, the model error variance by method of moments, and the final Lambda weighting matrix of cross correlation among basins
        end
       
        
        %Added by AGV 12/2/2011:Runs a constant GLS regression to determine model error variance for constant model
        aa=size(A,1);
        Ac=ones(aa,1);
        ExtrSc=[Ac(1:size(Ac,1),1:size(Ac,2)) B(1:size(B,1),1:size(B,2)) C(1:size(C,1),1:size(C,2)) E(1:size(E,1),1:size(E,2))];
        for ss=1:size(ExtrS,1)
            ExtrSc(ss,size(Ac,2)+size(B,2)+size(C,2)+size(E,2)+2)=Sigma(ss,1);
        end
       
        if UnSkew==1
            [ConstantFinalCoeff(:,1), ConstantSelGSQ, ConstantLambdaF]=NewLambdaMatrixFinalagvPRL(Ac,B,C,E,Sigma,m,dgeo,ExtrSc,Theta,Alpha,ChY,Conm,RT,RegionSkew,Exceed,GR,PRL); 
        else
            [ConstantFinalCoeff(:,1), ConstantSelGSQ, ConstantLambdaF]=LambdaMatrixFinalagvPRL(Ac,B,C,E,Sigma,m,dgeo,ExtrSc,Theta,Alpha,ChY,Conm,PRL);   %Calling out subroutine 'LambdaMatrix'
            %The output from this subroutine are the final coefficients, the model error variance by method of moments, and the final Lambda weighting matrix of cross correlation among basins
        end
        %End of section added by AGV
        
        %Residual vs fitted value plot
        for abca=1:size(A,1)
            CGLSSumYEST=0;    
            for abcde=1:size(A,2)
                CGLSSumYEST=CGLSSumYEST+(FinalCoeff(abcde+1,1)*A(abca, abcde));
            end
            CGLSOrigYEST(abca,1)=FinalCoeff(1,1)+CGLSSumYEST; %Estimated YHat
            CGLSOrigRes(abca,1)=B(abca, ChY)-CGLSOrigYEST(abca,1);     %Unscaled residual
            CGLSOrigResSq(abca,1)=CGLSOrigRes(abca,1)^2; %Squared residual
        end
        figure
        CGLSX=CGLSOrigYEST(:,1);
        CGLSY=CGLSOrigRes(:,1);
        plot(CGLSX,CGLSY,'ko')
        xlabel('Estimated Flow Characteristic (log base 10)','FontName','times','FontSize',14)
        ylabel('Residual (log base 10)','FontName','times','FontSize',14)
        title('Residuals versus Estimated Flow Characteristics','FontName','times','FontSize',14);
        box('on')   %places border on all four sides
        grid('on')
        
        %KE (2-27-08) Determining the standard error, T value, and P value for the coefficients
        XGLS=[ones(size(ExtrS,1),1) ExtrS(1:size(ExtrS,1), 1:size(A,2))];
        CCC=inv(XGLS'*inv(LambdaF)*XGLS);
        for jset=1:size(FinalCoeff,1)
            CoeffStdErr(jset,1)=(CCC(jset,jset))^0.5;   %Standard error on coefficients (ie square root of the the covariance term)
            TCoeff(jset,1)=FinalCoeff(jset,1)/CoeffStdErr(jset,1);   %t value for Ho of beta=0
            ProbTCoeff(jset,1)=2*(1-tcdf(abs(TCoeff(jset,1)),(size(A,1)-(size(A,2)+1))));    %Probability > |T|
        end
       
        %%%%%%%%%%%%%%%%%%%%
        %PERFORMANCE METRIC%
        %%%%%%%%%%%%%%%%%%%%
        %Average variance of prediction, AVP, is okay for regressions where 
        %each site shows up only once. Computing the average variance of 
        %prediction (AVP) used for each GLS regression model            
        %Computing the second term values of the average variance of 
        %prediction (AVP) metric used for each GLS regression model
        XAVP=[ones(size(ExtrS,1),1) ExtrS(1:size(ExtrS,1), 1:size(A,2))];
        for a=1:size(XAVP,1)
            Iteration=a
            xavp=[1 ExtrS(a,1:size(A,2))];
            Indsecterm(a,1)=xavp*inv(XAVP'*inv(LambdaF)*XAVP)*xavp';
        end
        AvgSecTerm=(1/size(ExtrS,1))*sum(Indsecterm(:,1));
        OrigAVP=SelGSQ+AvgSecTerm;
        
        %GLS leverage by Tasker and Stedinger, 1989
        LevX=[ones(size(A,1),1) A(:,:)];
        LevH=LevX*inv(LevX'*inv(LambdaF)*LevX)*LevX'*inv(LambdaF);
        for aa=1:size(LevH,1)
            LevGLS(aa,1)=LevH(aa,aa);
        end
        ConvGLSHlimit=(2*trace(LevH))/size(LevX,1);    %Leverage limit threshold. Leverage values of observations exceeding this value are considered "large leverage".
        
        %Figure for leverage values
        figure
        GXLe=linspace(1,size(A,1),size(A,1))';
        GYLe=LevGLS(:,1);
        GLim=(ones(size(GXLe,1),1)).*ConvGLSHlimit;
        plot(GXLe,GYLe,'ko',GXLe,GLim,'r-')
        xlabel('Observation','FontName','times','FontSize',14)
        ylabel('Leverage Value','FontName','times','FontSize',14)
        title('Leverage Values versus Observations','FontName','times','FontSize',14);
        box('on')   %places border on all four sides
        grid('on')
        
        %GLS influence by Tasker and Stedinger, 1989
        HPrime=LevX*inv(LevX'*inv(LambdaF)*LevX)*LevX';
        for ab=1:size(LevX,1)
            SumGLSYEST=0;
            for ac=1:size(A,2)
                SumGLSYEST=SumGLSYEST+(FinalCoeff(ac+1,1)*A(ab,ac));
            end
            OrigGLSYEST=FinalCoeff(1,1)+SumGLSYEST;
            ResGLSReg=B(ab,ChY)-OrigGLSYEST;    %Computing the regression residual at the ith basin
            
            CooksDGLS(ab,1)=(HPrime(ab,ab)*(ResGLSReg)^2)/(trace(LevH)*(LambdaF(ab,ab)-HPrime(ab,ab))^(2));
        end
        CooksDGLSlimit=4/size(LevX,1);    
        
        %Figure for influence values
        figure
        GXIn=linspace(1,size(A,1),size(A,1))';
        GYIn=CooksDGLS(:,1);
        GCLim=(ones(size(GXLe,1),1)).*CooksDGLSlimit;
        plot(GXIn,GYIn,'ko',GXIn,GCLim,'r-')
        xlabel('Observation','FontName','times','FontSize',14)
        ylabel('Influence Value','FontName','times','FontSize',14)
        title('Influence Values versus Observations','FontName','times','FontSize',14);
        box('on')   %places border on all four sides
        grid('on')
        
        %Pseudo R2 calcualtion- modified by AGV 12/2/2011
        %%Pseudo R2 calculation-explains the variance in the predictand based on the model error only! Conventional R2 base it on the TOTAL error (model+time-sampling errors).
        %XBasin2=[ones(size(A,1),1)];
        %YBasin=[B(1:size(B,1), ChY)];
        %PR2Coeff=regress(YBasin,XBasin2);
        %for kecda=1:size(B,1)
        %    PR2SqRes(kecda,1)=(B(kecda,ChY)-PR2Coeff)^2;
        %end
        %cc=mean(PR2SqRes(:,1))
        SelGSQ
        ConstantSelGSQ
        PR2=(1-(SelGSQ/ConstantSelGSQ))*100;  %Pseudo R2, corrected by AGV 12/2/2011
        % End of section modified by AGV 12/2/2011
        
        %Computation of the equivalent years of record for WLS
        AvgSigma2=mean(Sigma(:,1));
        AvgK2=mean(ExtrS(:,size(A,2)+size(B,2)+size(C,2)+2));
        Avgg2=mean(ExtrS(:,size(A,2)+size(B,2)+size(C,2)+3));
        Gcb=(AvgSigma2^2)*(1+(((AvgK2^2)/2)*(1+(0.75*(Avgg2^2))))+(AvgK2*Avgg2));
        EYR=(1/OrigAVP)*(Gcb);

        %JEK calculate MSE for GLS
        % 6.9.2011 
        SSres=sum(CGLSOrigResSq);
        MSE_GLS = SSres/(size(XGLS,1)-size(XGLS,2));
        
        %OUTPUT FILE FOR CONVENTIONAL GLS REGRESSION (PLACE IN REMNINDER TO PRINT IN "LANDSCAPE" AND NOT "PORTRAIT")
        kecdabc=1;
        for kecdab=1:size(LevGLS,1)
            SizeLevGLS(kecdab,1)=kecdabc;
            SizeCooksDGLS(kecdab,1)=kecdabc;
            kecdabc=kecdabc+1;
        end
        COEFFINFO1=[FinalCoeff(:,1) CoeffStdErr(:,1) TCoeff(:,1) ProbTCoeff(:,1)];
        COEFFINFO2=[FinalCoeff(2:size(COEFFINFO1,1),1)'; CoeffStdErr(2:size(COEFFINFO1,1),1)'; TCoeff(2:size(COEFFINFO1,1),1)'; ProbTCoeff(2:size(COEFFINFO1,1),1)'];
        LevInf=[SizeLevGLS(:,1)'; CGLSOrigRes(:,1)'; LevGLS(:,1)'; CooksDGLS(:,1)'];
        VPprint=[SizeLevGLS(:,1)'; B(:,ChY)'; CGLSOrigYEST(:,1)'; Indsecterm(:,1)'+SelGSQ]; %JEK added
        OrigAVP1=100*((exp(5.302*OrigAVP)-1)^(0.5));
        SelGSQ2=100*((exp(5.302*SelGSQ)-1)^(0.5));

        fid=fopen('ConventionalGLS.txt','w');
        fprintf(fid, 'WREG v%s \n', versionnum); % print version number in header
        fprintf(fid, '%s \n\n', versiondate); % print version date
        fprintf(fid,'Regression Model for %s\n',Name_B_Selected);
        fprintf(fid,'                                                     \n');
        fprintf(fid,'Performance Metrics                                  \n');
        fprintf(fid,'MSE of unweighted residuals      %10.3f (log units)\n', MSE_GLS);
        fprintf(fid,'Avg Variance of Prediction       %10.3f (log units)\n',OrigAVP);
        fprintf(fid,'Avg Standard Error of Prediction %10.2f (percent)\n',OrigAVP1);
        fprintf(fid,'Pseudo R2                        %10.2f (percent)    \n',PR2);
        %fprintf(fid,'Effective Years of Record        %10.2f          \n',EYR);
        fprintf(fid,'Model Error Variance             %10.3f (log units) \n',SelGSQ);
        fprintf(fid,'Standard Model Error             %10.2f (percent)\n',SelGSQ2);
        %fprintf(fid,'*Note: Avg Standard Error of Prediction, Pseudo R2,  \n');
        %fprintf(fid,'and Standard Model Error are in percent.             \n');
        %fprintf(fid,'Years of Record are in years.                        \n');
        fprintf(fid,'                                                     \n');
        fprintf(fid,'Coefficients of Model                                \n');
        fprintf(fid,'                        Std Error  T value   P>|T|   \n');
        fprintf(fid,'Constant    %10.3f     %6.2f   %6.2f  %6.3f\n',COEFFINFO1(1,:));
%IC        fprintf(fid,'Coefficient %10.3f     %6.2f   %6.2f  %6.3f\n',COEFFINFO2);
        n_space=12-size(Name_A_Selected,2);
        text_space='';
        for ispace=1:n_space
            text_space=[text_space ' '];
        end
        for iprint=1:N_A_Selected
            fprintf(fid,'%s%s%10.3f     %6.2f   %6.2f  %6.3f\n',Name_A_Selected(iprint,:),text_space,COEFFINFO2(:,iprint));
        end
        fprintf(fid,'                                                   \n');
        fprintf(fid,'Transformations           C1      C2      C3      C4\n');
        n_space=12-size(Name_B_Selected,2);
        text_space='';
        for ispace=1:n_space
            text_space=[text_space ' '];
        end
        switch B_Transform
            case 1
                Tran_text='       1';
            case 2
                Tran_text='   log10';
            case 3
                Tran_text='      ln';
            case 4
                Tran_text='     exp';
        end
        fprintf(fid,'%s%s%s%8.2f%8.2f%8.2f%8.2f\n',Name_B_Selected(1,:),text_space,Tran_text,B_Constant(1,:));
        n_space=12-size(Name_A_Selected,2);
        text_space='';
        for ispace=1:n_space
            text_space=[text_space ' '];
        end
        for iprint=1:N_A_Selected
            switch A_Transform(iprint)
                case 1
                    Tran_text='       1';
                case 2
                    Tran_text='   log10';
                case 3
                    Tran_text='      ln';
                case 4
                    Tran_text='     exp';
            end
            fprintf(fid,'%s%s%s%8.2f%8.2f%8.2f%8.2f\n',Name_A_Selected(iprint,:),text_space,Tran_text,A_Constant(iprint,:));
        end
        fprintf(fid,'                                                     \n');
        fprintf(fid,'Residuals, Leverage, and Influence of Observations   \n');
        fprintf(fid,'Leverage Limit                   %10.3f          \n',ConvGLSHlimit);
        fprintf(fid,'Influence Limit                  %10.3f          \n',CooksDGLSlimit);
        fprintf(fid,'             Observation  Residual Leverage Influence\n');
        fprintf(fid,'%23.0f%10.3f%10.3f%10.3f\n',LevInf(:,:));
        fprintf(fid,'\n');
        fprintf(fid,'Variance of prediction for streamgages used in regression\n');
        fprintf(fid,'             Observation  ObsValue  PredValue VarianceofPrediction\n');
        fprintf(fid,'%23.0f %10.3f %10.3f %10.3f\n',VPprint);
        %fprintf(fid,'*Note: All 9999 indicate no logistic regession available. All -9999 indicate N/A. Regr.       \n');
        fclose(fid);
        
        save InvXLX.txt CCC -ASCII
    % ---------------------------------------------------------------------
    elseif ParFit==2    %WLS parameter fitting
    % ---------------------------------------------------------------------
    %GLOBAL Sigma Regression, so using global parameter values instead of localized RoI ones (Regressing by multiple linear regression between each stations std dev on annual peaks to corresponding basin attributes)
        GlobalPreds=[A(1:size(A,1), 1:size(A,2))];
        GXBasin=[ones(size(GlobalPreds,1),1) GlobalPreds(1:size(GlobalPreds,1), 1:size(A,2))];  %Predictors at each selected station
        GLSigma=[E(:,1)];                                                                       %Lsigma values
        SigCoeff=regress(GLSigma,GXBasin);
        
        ExtrS=[A(1:size(A,1),1:size(A,2)) B(1:size(B,1),1:size(B,2)) C(1:size(C,1),1:size(C,2)) E(1:size(E,1),1:size(E,2))];
        %The 'ExtrS' matrixs contains the following info basin attributes, predictands, lat and long, at-site Lsigma, K, weighted skew, estimates sigma values for all the basins in the RoI
        
        %Calculating the sample Std Dev, si, at each gauged basin in the RoI (Using si as estimator of sigma i)
        for ffff=1:size(A,1)
            SumSigma=0;
            for fefe=1:size(A,2)
                SumSigma=SumSigma+(SigCoeff(fefe+1,1)*A(ffff,fefe));
            end
            Sigma(ffff,1)=SigCoeff(1,1)+SumSigma;
        end
        
        for ssss=1:size(ExtrS,1)
            ExtrS(ssss,size(A,2)+size(B,2)+size(C,2)+size(E,2)+2)=Sigma(ssss,1);
        end
        
        if UserWeight==999
        %Calculating weighting function (wWLS=1/(ca+cb*(1/n)))
        AvgSigma=mean(Sigma(:,1));
        AvgK=mean(ExtrS(:,size(A,2)+size(B,2)+size(C,2)+2));
        Avgg=mean(ExtrS(:,size(A,2)+size(B,2)+size(C,2)+3));
        cb=(AvgSigma^2)*(1+(((AvgK^2)/2)*(1+(0.75*(Avgg^2))))+(AvgK*Avgg));
        if cb<0
            cb=0;
        else
                
        end
        
        %Calculating the observed standard error of estimate using OLS
        XOLS2=[ones(size(ExtrS,1),1) ExtrS(1:size(ExtrS,1), 1:size(A,2))];
        YOLS2=[ExtrS(1:size(ExtrS,1), size(A,2)+ChY)];
        ZOLS2=regress(YOLS2,XOLS2); 
        FCoeff(:,1)=ZOLS2;
        for aaaa=1:size(XOLS2,1)
            SumOLS2=0;
            for bbbb=1:size(A,2)
                SumOLS2=SumOLS2+(FCoeff(bbbb+1,1)*XOLS2(aaaa,bbbb+1));
            end
            YESTOLS2(aaaa,1)=FCoeff(1,1)+SumOLS2;
            ResOLS2(aaaa,1)=YOLS2(aaaa,1)-YESTOLS2(aaaa,1);
            ResSQOLS2(aaaa,1)=(YOLS2(aaaa,1)-YESTOLS2(aaaa,1))^2;
        end        
        ObsVar(1,1)=(sum(ResSQOLS2(:,1))/(size(XOLS2,1)-size(A,2)-1));
        
        
        %CONSTANT MODEL (added by AGV 12/4/2011)- Calculating the observed standard error of estimate using OLS
        XOLS2constant=[ones(size(ExtrS,1),1)];
        YOLS2=[ExtrS(1:size(ExtrS,1), size(A,2)+ChY)];
        ZOLS2constant=regress(YOLS2,XOLS2constant); 
        FCoeffConstant(:,1)=ZOLS2constant;
        YESTOLS2constant=ones(size(XOLS2constant,1),size(XOLS2constant,2)).* FCoeffConstant;
        ResOLS2constant=YOLS2-YESTOLS2constant;
        ResSQOLS2constant=(YOLS2-YESTOLS2constant).^2;
        ConstantObsVar(1,1)=(sum(ResSQOLS2constant(:,1))/(size(XOLS2constant,1)-1));
        %end of section added by AGV
        
        
        %Calculating Avg Inverse Record Lengths from POR, mWLS
        %Calculating Record Lengths from POR, m
        for fff=1:size(SIZELF2,1)    
            m(fff,1)=SIZELF2(fff,1);
            mWLS(fff,1)=(m(fff,1))^-1;
        end
        for hhh=1:size(ExtrS,1)
            ExtrS(hhh,1+size(A,2)+size(B,2)+size(C,2)+size(E,2)+1+size(Sigma,2))=mWLS(hhh,1);
        end
        Avgn=mean(mWLS(:,1));
            
        ca=ObsVar(1,1)-(cb*Avgn);
        if ca<0
            ca=0;
        else
                
        end
        
        %CONSTANT MODEL ca -added by AGV 12/4/2011(model error variance) for constant model
        caCONSTANT=ConstantObsVar(1,1)-(cb*Avgn);
        if caCONSTANT<0
            caCONSTANT=0;
        else
        end
        %end of section added by AGV
        
        for wwww=1:size(ExtrS,1)
            wWLS(wwww,1)=(ca+cb*(mWLS(wwww,1)));   %A vector of the weights. In the paper this quantity is raised to the -1 power, but I handle that later when I take the inverse of it.
        end    
        Ident=eye(size(wWLS,1));
        for xxxx=1:size(Ident,1)
            for yyyy=1:size(Ident,1)
                if xxxx==yyyy
                    Wmatrix2(xxxx,yyyy)=wWLS(xxxx,1);
                else
                        
                end
            end
        end
        
        Wmatrix = Wmatrix2;
        %JEK 11.09.09 
        %------------------
        % deleted temporary code normalizing matrix.  Wmatrix 
        % (for Tasker weights) does not need to
        % be normalized because values within it are variances.  if weights
        % are assigned more arbitrarily, as with the userWLS, they must be
        % normalized and variance must be accounted for separately in
        % various equations.
        
        else    %uses the users weighting matrix (input in terms of variance)
            Wmatrix=UW;
        end    
        XBasin=[ones(size(A,1),1) A(1:size(A,1), 1:size(A,2))];
        YBasin=[B(1:size(B,1), ChY)];
        ZWLS=inv(XBasin'*inv(Wmatrix)*XBasin)*XBasin'*inv(Wmatrix)*YBasin;  
        FinalCoeff(:,1)=ZWLS;
        
        %KE (12-23-08) Providing the sum of squared residual to compute the
        %model error variance (It assumes that total error=model error for user specified weighting matrices).
        if UserWeight==999 % default WLS
            %JEK 11.05.09 Also calculate and save for default WLS
            SSres=((YBasin-XBasin*ZWLS)')*(YBasin-XBasin*ZWLS);
            save SSres.txt SSres -ASCII
            SStot=(YBasin'*YBasin)-(1/size(YBasin,1))*(sum(YBasin(:,1)))^(2);
            save SStot.txt SStot -ASCII
        else %User specified WLS
            SSres=((YBasin-XBasin*ZWLS)')*(YBasin-XBasin*ZWLS);
            save SSres.txt SSres -ASCII
            SStot=(YBasin'*YBasin)-(1/size(YBasin,1))*(sum(YBasin(:,1)))^(2);
            save SStot.txt SStot -ASCII
        end
        
        %Residual vs fitted value plot
        for abcb=1:size(A,1)
            CWLSSumYEST=0;    
            for abcdf=1:size(A,2)
                CWLSSumYEST=CWLSSumYEST+(FinalCoeff(abcdf+1,1)*A(abcb, abcdf));
            end
            CWLSOrigYEST(abcb,1)=FinalCoeff(1,1)+CWLSSumYEST; %Estimated YHat
            CWLSOrigRes(abcb,1)=B(abcb, ChY)-CWLSOrigYEST(abcb,1);     %Unscaled residual
            CWLSOrigResSq(abcb,1)=CWLSOrigRes(abcb,1)^2;
        end
        figure
        CWLSX=CWLSOrigYEST(:,1);
        CWLSY=CWLSOrigRes(:,1);
        plot(CWLSX,CWLSY,'ko')
        xlabel('Estimated Flow Characteristic (log base 10)','FontName','times','FontSize',14)
        ylabel('Residual (log base 10)','FontName','times','FontSize',14)
        title('Residuals versus Estimated Flow Characteristics','FontName','times','FontSize',14);
        box('on')   %places border on all four sides
        grid('on')
        
        %KE (2-27-08) Determining the standard error, T value, and P value for the coefficients
        if UserWeight==999 %standard Tasker WLS
            XGLS=[ones(size(ExtrS,1),1) ExtrS(1:size(ExtrS,1), 1:size(A,2))];
            CCC=inv(XGLS'*inv(Wmatrix)*XGLS);
            %JEK 11.05.09 add MSE to calc of coeff std error
            weightedresids =sqrt(inv(Wmatrix)*CWLSOrigResSq);
            weightedMSE = sum(weightedresids.^2)/(size(XGLS,1)-size(XGLS,2));
            MSE_WLS = SSres/(size(XGLS,1)-size(XGLS,2));
            for jset=1:size(FinalCoeff,1)
                CoeffStdErr(jset,1)=(CCC(jset,jset))^0.5;   %Standard error on coefficients (ie square root of the the covariance term)
                %JEK 11.05.09 - new CoeffStdErr including MSE - not needed
                %because CCC includes the weighting matrix, in terms of
                %variance
                %CoeffStdErr(jset,1)=sqrt(weightedMSE*CCC(jset,jset));
                TCoeff(jset,1)=FinalCoeff(jset,1)/CoeffStdErr(jset,1);   %t value for Ho of beta=0
                ProbTCoeff(jset,1)=2*(1-tcdf(abs(TCoeff(jset,1)),(size(A1,1)-size(A1,2)-1)));    %Probability > |T|
            end
            save InvXLX.txt CCC -ASCII
        else    %User defined matrix
            XGLS=[ones(size(ExtrS,1),1) ExtrS(1:size(ExtrS,1), 1:size(A,2))];
            % JEK 11.18.09next line is incorrect
            %SSRESWLS=(YBasin'*YBasin)-(FinalCoeff'*XGLS'*YBasin);
            % so change OLSSigmaWLS equation to use SSres
            OLSSigmaWLS=SSres/(size(A,1)-(size(A,2)+1));
            rawMSE = OLSSigmaWLS;

            weightedresids =sqrt(inv(Wmatrix)*CWLSOrigResSq);
            weightedMSE = sum(weightedresids.^2)/(size(XGLS,1)-size(XGLS,2));
            %SSRESCTWLS=size(A,1)-(size(A1,2)+1);
            % jek 11.10.09 add Wmatrix to CCC.  But because it is normalized,
            % still need to multiply by MSE to calculate coeffstderr
            % using weightedMSE
            CCCWLS=inv(XGLS'*inv(Wmatrix)*XGLS);  %% THIS IS InvXLX %%%
            for jsetw=1:size(FinalCoeff,1)
                CoeffStdErr(jsetw,1)=(weightedMSE*CCCWLS(jsetw,jsetw))^0.5;   %Standard error on coefficients (ie square root of the the covariance term)
                TCoeff(jsetw,1)=FinalCoeff(jsetw,1)/CoeffStdErr(jsetw,1);   %t value for Ho of beta=0
                ProbTCoeff(jsetw,1)=2*(1-tcdf(abs(TCoeff(jsetw,1)),(size(A,1)-(size(A,2)+1))));    %Probability > |T|
            end
            
            MSECCCWLS = weightedMSE.*CCCWLS;
            save InvXLX.txt MSECCCWLS -ASCII %JEK 6.9.2011 Print InvXLX for user-defined WLS
               %must be multiplied by the weightedMSE because the
               %user-defined weights are not variance-based.
               % results are very slightly different from std - not sure why -
               % double precision output of WMatrix from TaskerWLS didn't
               % help but it seems to be just a rounding issue JEK 2.9.2012
        end
      
        if UserWeight==999
        %Average variance of prediction, AVP, is okay for regressions where each site shows up only once. Computing the average variance of prediction (AVP) used for each WLS regression model            
        %Computing the second term values of the average variance of prediction (AVP) metric used for each GLS regression model
        %This represents what we would get if we had ungaged basins that had very similar basin attributes placed into the model.
            XAVP=[ones(size(ExtrS,1),1) ExtrS(1:size(ExtrS,1), 1:size(A,2))];
            for a=1:size(XAVP,1)
                Iteration=a
                xavp=[1 ExtrS(a,1:size(A,2))];
                Indsecterm(a,1)=xavp*inv(XAVP'*inv(Wmatrix)*XAVP)*xavp';
            end
            AvgSecTerm=(1/size(ExtrS,1))*sum(Indsecterm(:,1));
            OrigAVP=ca+AvgSecTerm;
            %modelerrorvarianceJEK = ca
        else
            % need to recheck this calculation!!
            XAVP=[ones(size(ExtrS,1),1) ExtrS(1:size(ExtrS,1), 1:size(A,2))];
            for a=1:size(XAVP,1)
                Iteration=a
                xavp=[1 ExtrS(a,1:size(A,2))];
                Indsecterm(a,1)=OLSSigmaWLS*xavp*inv(XAVP'*inv(Wmatrix)*XAVP)*xavp';
            end
            AvgSecTerm=(1/size(ExtrS,1))*sum(Indsecterm(:,1));
            OrigAVP=OLSSigmaWLS + AvgSecTerm;
        end
        %An equivalent WLS leverage modeled after the one by Tasker and Stedinger, 1989
        LevX=[ones(size(A,1),1) A(:,:)];
        WLSLevH=LevX*inv(LevX'*inv(Wmatrix)*LevX)*LevX'*inv(Wmatrix);
        for aaa=1:size(WLSLevH,1)
            LevWLS(aaa,1)=WLSLevH(aaa,aaa);
        end
        ConvWLSHlimit=(2*trace(WLSLevH))/size(LevX,1);    %Leverage limit threshold. Leverage values of observations exceeding this value are considered "large leverage".
        
        %Figure for leverage values
        figure
        WXLe=linspace(1,size(A,1),size(A,1))';
        WYLe=LevWLS(:,1);
        WLim=(ones(size(WXLe,1),1)).*ConvWLSHlimit;
        plot(WXLe,WYLe,'ko',WXLe,WLim,'r-')
        xlabel('Observation','FontName','times','FontSize',14)
        ylabel('Leverage Value','FontName','times','FontSize',14)
        title('Leverage Values versus Observations','FontName','times','FontSize',14);
        box('on')   %places border on all four sides
        grid('on')
        
        %An equivalent WLS influence modeled after the one by Tasker and Stedinger, 1989
        HPrime=LevX*inv(LevX'*inv(Wmatrix)*LevX)*LevX';
        for abb=1:size(LevX,1)
            SumWLSYEST=0;
            for acc=1:size(A,2)
                SumWLSYEST=SumWLSYEST+(FinalCoeff(acc+1,1)*A(abb,acc));
            end
            OrigWLSYEST=FinalCoeff(1,1)+SumWLSYEST;
            ResWLSReg=B(abb,ChY)-OrigWLSYEST;    %Computing the regression residual at the ith basin
            
            CooksDWLS(abb,1)=(HPrime(abb,abb)*(ResWLSReg)^2)/(trace(WLSLevH)*(Wmatrix(abb,abb)-HPrime(abb,abb))^(2));
        end
        CooksDWLSlimit=4/size(LevX,1);  
        
        %Figure for influence values
        figure
        WXIn=linspace(1,size(A,1),size(A,1))';
        WYIn=CooksDWLS(:,1);
        WCLim=(ones(size(WXLe,1),1)).*CooksDWLSlimit;
        plot(WXIn,WYIn,'ko',WXIn,WCLim,'r-')
        xlabel('Observation','FontName','times','FontSize',14)
        ylabel('Influence Value','FontName','times','FontSize',14)
        title('Influence Values versus Observations','FontName','times','FontSize',14);
        box('on')   %places border on all four sides
        grid('on')
        
        %Pseudo R2 modified by AGV 12/4/2011
        %Pseudo R2 calculation-explains the variance in the predictand based on the model error only! Conventional R2 base it on the TOTAL error (model+time-sampling errors).
        if UserWeight==999
            %XBasin2=[ones(size(A,1),1)];
            %PR2Coeff=regress(YBasin,XBasin2);
            %for kecd=1:size(B,1)
            %    PR2SqRes(kecd,1)=(B(kecd,ChY)-PR2Coeff)^2;
            %end
            %cc=mean(PR2SqRes(:,1));
            PR2=(1-(ca/caCONSTANT))*100;  %Pseudo R2
            %end section modified by AGV
            
            %Computation of the equivalent years of record for WLS
            EYR=(1/OrigAVP)*(cb);
        else
        end
        %end of section modified by AGV
        
        %OUTPUT FILE FOR CONVENTIONAL WLS REGRESSION (PLACE IN REMNINDER TO PRINT IN "LANDSCAPE" AND NOT "PORTRAIT")
        if UserWeight==999
            kecdabc=1;
            for kecdab=1:size(LevWLS,1)
                SizeLevWLS(kecdab,1)=kecdabc;
                SizeCooksDWLS(kecdab,1)=kecdabc;
                kecdabc=kecdabc+1;
            end
            COEFFINFO1=[FinalCoeff(:,1) CoeffStdErr(:,1) TCoeff(:,1) ProbTCoeff(:,1)];
            COEFFINFO2=[FinalCoeff(2:size(COEFFINFO1,1),1)'; CoeffStdErr(2:size(COEFFINFO1,1),1)'; TCoeff(2:size(COEFFINFO1,1),1)'; ProbTCoeff(2:size(COEFFINFO1,1),1)'];
            LevInf=[SizeLevWLS(:,1)'; CWLSOrigRes(:,1)'; LevWLS(:,1)'; CooksDWLS(:,1)'];
            VPprint=[SizeLevWLS(:,1)'; B(:,ChY)'; CWLSOrigYEST(:,1)'; Indsecterm(:,1)'+ca];
            OrigAVP1=100*((exp(5.302*OrigAVP)-1)^(0.5));
            ca2=100*((exp(5.302*ca)-1)^(0.5));
        
            fid=fopen('ConventionalWLS.txt','w');
            fprintf(fid, 'WREG v%s\n', versionnum); % print version number in header
            fprintf(fid, '%s\n \n', versiondate); % print version date
            fprintf(fid,'Regression Model for %s\n',Name_B_Selected);
            fprintf(fid,'                                                     \n');
            fprintf(fid,'Performance Metrics                                  \n');
            fprintf(fid,'MSE of unweighted residuals      %10.3f (log units)\n', MSE_WLS);
            fprintf(fid,'Avg Variance of Prediction       %10.3f (log units)\n',OrigAVP);
            fprintf(fid,'Avg Standard Error of Prediction %10.2f (percent)\n',OrigAVP1);
            fprintf(fid,'Pseudo R2                        %10.2f (percent)  \n',PR2);
            %fprintf(fid,'Effective Years of Record        %10.2f          \n',EYR);
            fprintf(fid,'Model Error Variance             %10.3f (log units)\n',ca);
            fprintf(fid,'Standard Model Error             %10.2f (percent)\n',ca2);
            %fprintf(fid,'Years of Record are in years.                        \n');
            fprintf(fid,'                                                     \n');
            fprintf(fid,'Coefficients of Model                                \n');
            fprintf(fid,'                        Std Error  T value   P>|T|   \n');
            fprintf(fid,'Constant    %10.3f     %6.2f   %6.2f  %6.3f\n',COEFFINFO1(1,:));
%IC            fprintf(fid,'Coefficient %10.2f     %6.2f   %6.2f  %6.3f\n',COEFFINFO2);
            n_space=12-size(Name_A_Selected,2);
            text_space='';
            for ispace=1:n_space
                text_space=[text_space ' '];
            end
            for iprint=1:N_A_Selected
                fprintf(fid,'%s%s%10.3f     %6.2f   %6.2f  %6.3f\n',Name_A_Selected(iprint,:),text_space,COEFFINFO2(:,iprint));
            end
            fprintf(fid,'                                                    \n');
            fprintf(fid,'Transformations           C1      C2      C3      C4\n');
            n_space=12-size(Name_B_Selected,2);
            text_space='';
            for ispace=1:n_space
                text_space=[text_space ' '];
            end
            switch B_Transform
                case 1
                    Tran_text='       1';
                case 2
                    Tran_text='   log10';
                case 3
                    Tran_text='      ln';
                case 4
                    Tran_text='     exp';
            end
            fprintf(fid,'%s%s%s%8.2f%8.2f%8.2f%8.2f\n',Name_B_Selected(1,:),text_space,Tran_text,B_Constant(1,:));
            n_space=12-size(Name_A_Selected,2);
            text_space='';
            for ispace=1:n_space
                text_space=[text_space ' '];
            end
            for iprint=1:N_A_Selected
                switch A_Transform(iprint)
                    case 1
                        Tran_text='       1';
                    case 2
                        Tran_text='   log10';
                    case 3
                        Tran_text='      ln';
                    case 4
                        Tran_text='     exp';
                end
                fprintf(fid,'%s%s%s%8.2f%8.2f%8.2f%8.2f\n',Name_A_Selected(iprint,:),text_space,Tran_text,A_Constant(iprint,:));
            end
            fprintf(fid,'                                                     \n');
            fprintf(fid,'Residuals, Leverage, and Influence of Observations   \n');
            fprintf(fid,'Leverage Limit                   %10.3f          \n',ConvWLSHlimit);
            fprintf(fid,'Influence Limit                  %10.3f          \n',CooksDWLSlimit);
            fprintf(fid,'             Observation  Residual Leverage Influence\n');
            fprintf(fid,'%23.0f%10.3f%10.3f%10.3f\n',LevInf(:,:));
            fprintf(fid,'\n');
            fprintf(fid,'Variance of prediction for streamgages used in regression\n');
            fprintf(fid,'             Observation  ObsValue  PredValue  VarianceofPrediction\n');
            fprintf(fid,'%23.0f %10.3f %10.3f %10.3f\n',VPprint);
            %fprintf(fid,'*Note: All 9999 indicate no logistic regession available. All -9999 indicate N/A. Regr.       \n');
            fclose(fid);
        else % user defined WLS
            kecdabc=1;
            for kecdab=1:size(LevWLS,1)
                SizeLevWLS(kecdab,1)=kecdabc;
                SizeCooksDWLS(kecdab,1)=kecdabc;
                kecdabc=kecdabc+1;
            end
            COEFFINFO1=[FinalCoeff(:,1) CoeffStdErr(:,1) TCoeff(:,1) ProbTCoeff(:,1)];
            COEFFINFO2=[FinalCoeff(2:size(COEFFINFO1,1),1)'; CoeffStdErr(2:size(COEFFINFO1,1),1)'; TCoeff(2:size(COEFFINFO1,1),1)'; ProbTCoeff(2:size(COEFFINFO1,1),1)'];
            LevInf=[SizeLevWLS(:,1)'; CWLSOrigRes(:,1)'; LevWLS(:,1)'; CooksDWLS(:,1)'];
            VPprint=[SizeLevWLS(:,1)'; B(:,ChY)'; CWLSOrigYEST(:,1)'; Indsecterm(:,1)'+OLSSigmaWLS];
            OrigAVP1=100*((exp(5.302*OrigAVP)-1)^(0.5));
            RMSEpercent = 100*((exp(5.302*OLSSigmaWLS)-1)^(0.5));
            %ca2=100*((exp(5.302*ca)-1)^(0.5));
        
            fid=fopen('ConventionalWLS.txt','w');
            fprintf(fid, 'WREG v%s\n', versionnum); % print version number in header
            fprintf(fid, '%s\n \n', versiondate); % print version date
            fprintf(fid,'Regression Model for %s\n',Name_B_Selected);
            fprintf(fid,'A USER SPECIFIED WEIGHTING MATRIX USED.              \n');
            fprintf(fid,'                                                     \n');
            fprintf(fid,'Performance Metrics                                  \n');
            fprintf(fid,'MSE of unweighted residuals       %10.3f (log units)\n', OLSSigmaWLS);
            fprintf(fid,'Avg Variance of Prediction        %10.3f (log units)\n', OrigAVP);
            fprintf(fid,'Avg Standard Error of Prediction  %10.2f (percent)\n', OrigAVP1);
            fprintf(fid,'    (VP and SEP are calculated using MSE-residuals)\n');
            %fprintf(fid,'Pseudo R2                            NA              \n');
            %fprintf(fid,'Effective Years of Record            NA              \n');
            fprintf(fid,'RMSE-percent                      %10.2f\n', RMSEpercent);
            %fprintf(fid,'Years of Record are in years.                        \n');
            fprintf(fid,'                                                     \n');
            fprintf(fid,'Coefficients of Model                                \n');
            fprintf(fid,'                        Std Error  T value   P>|T|   \n');
            fprintf(fid,'Constant    %10.3f     %6.2f   %6.2f  %6.3f\n',COEFFINFO1(1,:));
%IC            fprintf(fid,'Coefficient %10.2f     %6.2f   %6.2f  %6.3f\n',COEFFINFO2);
            n_space=12-size(Name_A_Selected,2);
            text_space='';
            for ispace=1:n_space
                text_space=[text_space ' '];
            end
            for iprint=1:N_A_Selected
                fprintf(fid,'%s%s%10.3f     %6.2f   %6.2f  %6.3f\n',Name_A_Selected(iprint,:),text_space,COEFFINFO2(:,iprint));
            end
            fprintf(fid,'                                                   \n');
            fprintf(fid,'Transformations           C1      C2      C3      C4\n');
            n_space=12-size(Name_B_Selected,2);
            text_space='';
            for ispace=1:n_space
                text_space=[text_space ' '];
            end
            switch B_Transform
                case 1
                    Tran_text='       1';
                case 2
                    Tran_text='   log10';
                case 3
                    Tran_text='      ln';
                case 4
                    Tran_text='     exp';
            end
            fprintf(fid,'%s%s%s%8.2f%8.2f%8.2f%8.2f\n',Name_B_Selected(1,:),text_space,Tran_text,B_Constant(1,:));
            n_space=12-size(Name_A_Selected,2);
            text_space='';
            for ispace=1:n_space
                text_space=[text_space ' '];
            end
            for iprint=1:N_A_Selected
                switch A_Transform(iprint)
                    case 1
                        Tran_text='       1';
                    case 2
                        Tran_text='   log10';
                    case 3
                        Tran_text='      ln';
                    case 4
                        Tran_text='     exp';
                end
                fprintf(fid,'%s%s%s%8.2f%8.2f%8.2f%8.2f\n',Name_A_Selected(iprint,:),text_space,Tran_text,A_Constant(iprint,:));
            end
            fprintf(fid,'                                                     \n');
            fprintf(fid,'Residuals, Leverage, and Influence of Observations   \n');
            fprintf(fid,'Leverage Limit                   %10.3f          \n',ConvWLSHlimit);
            fprintf(fid,'Influence Limit                  %10.3f          \n',CooksDWLSlimit);
            fprintf(fid,'             Observation  Residual Leverage Influence\n');
            fprintf(fid,'%23.0f%10.3f%10.3f%10.3f\n',LevInf(:,:));
            fprintf(fid,'\n');
            fprintf(fid,'Variance of prediction for streamgages used in regression\n');
            fprintf(fid,'             Observation  ObsValue  PredValue  VarianceofPrediction\n');
            fprintf(fid,'%23.0f %10.3f %10.3f %10.3f\n',VPprint);
            %fprintf(fid,'*Note: All 9999 indicate no logistic regession available. All -9999 indicate N/A. Regr.       \n');
            fclose(fid);
        end 
 
        save Wmatrix.txt Wmatrix -ASCII -double -tabs %added double precision and tab delimited JEK 2.9.2012
        
    else %ParFit==1 %OLS parameter fitting
        XOLS=[ones(size(A,1),1) A(1:size(A,1), 1:size(A,2))];
        YOLS=[B(1:size(B,1), ChY)];
        ZOLS=regress(YOLS,XOLS); 
        FinalCoeff(:,1)=ZOLS;
        
        %KE (2-27-08) Determining the standard error, T value, and P value for the coefficients
        SSRES=(YOLS'*YOLS)-(FinalCoeff'*XOLS'*YOLS);
        OLSSigma=((YOLS'*YOLS)-(FinalCoeff'*XOLS'*YOLS))/(size(A,1)-(size(A,2)+1));
        SSRESCT=size(A,1)-(size(A1,2)+1);
        CCC=inv(XOLS'*XOLS);
        for jset=1:size(FinalCoeff,1)
            CoeffStdErr(jset,1)=(OLSSigma*CCC(jset,jset))^0.5;   %Standard error on coefficients (ie square root of the the covariance term)
            TCoeff(jset,1)=FinalCoeff(jset,1)/CoeffStdErr(jset,1);   %t value for Ho of beta=0
            ProbTCoeff(jset,1)=2*(1-tcdf(abs(TCoeff(jset,1)),(size(A,1)-(size(A,2)+1))));    %Probability > |T|
        end
        
        %save A.txt A -ASCII
        %save A1.txt A1 -ASCII
        %save SSRESCT.txt SSRESCT -ASCII
        %save SSRES.txt SSRES -ASCII
        %save OLSSigma.txt OLSSigma -ASCII
        %save CCC.txt CCC -ASCII
        
        %Residual vs fitted value plot
        for abcc=1:size(A,1)
            COLSSumYEST=0;    
            for abcdg=1:size(A,2)
                COLSSumYEST=COLSSumYEST+(FinalCoeff(abcdg+1,1)*A(abcc, abcdg));
            end
            COLSOrigYEST(abcc,1)=FinalCoeff(1,1)+COLSSumYEST; %Estimated YHat
            COLSOrigRes(abcc,1)=B(abcc, ChY)-COLSOrigYEST(abcc,1);     %Unscaled residual
            COLSOrigResSq(abcc,1)=COLSOrigRes(abcc,1)^2;
        end
        figure
        COLSX=COLSOrigYEST(:,1);
        COLSY=COLSOrigRes(:,1);
        plot(COLSX,COLSY,'ko')
        xlabel('Estimated Flow Characteristic (log base 10)','FontName','times','FontSize',14)
        ylabel('Residual (log base 10)','FontName','times','FontSize',14)
        title('Residuals versus Estimated Flow Characteristics','FontName','times','FontSize',14);
        box('on')   %places border on all four sides
        grid('on')
        
        %OLS leverage
        LevX=[ones(size(A,1),1) A(:,:)];
        OLSLevH=LevX*inv(LevX'*LevX)*LevX';
        for aaa=1:size(OLSLevH,1)
            LevOLS(aaa,1)=OLSLevH(aaa,aaa);
        end
        ConvOLSHlimit=(2*trace(OLSLevH))/size(LevX,1);    %Leverage limit threshold. Leverage values of observations exceeding this value are considered "large leverage".
        
        %Figure for leverage values
        figure
        OXLe=linspace(1,size(A,1),size(A,1))';
        OYLe=LevOLS(:,1);
        OLim=(ones(size(OXLe,1),1)).*ConvOLSHlimit;
        plot(OXLe,OYLe,'ko',OXLe,OLim,'r-')
        xlabel('Observation','FontName','times','FontSize',14)
        ylabel('Leverage Value','FontName','times','FontSize',14)
        title('Leverage Values versus Observations','FontName','times','FontSize',14);
        box('on')   %places border on all four sides
        grid('on')
        
        %OLS influence
        for abbb=1:size(LevX,1)
            SumOLSYEST=0;
            for accc=1:size(A,2)
                SumOLSYEST=SumOLSYEST+(FinalCoeff(accc+1,1)*A(abbb,accc));
            end
            OrigOLSYEST=FinalCoeff(1,1)+SumOLSYEST;
            ResOLSReg=B(abbb,ChY)-OrigOLSYEST;    %Computing the regression residual at the ith basin
            
            CooksDOLS(abbb,1)=(OLSLevH(abbb,abbb)*(ResOLSReg)^2)/(trace(OLSLevH)*(1-OLSLevH(abbb,abbb))^(2));
        end
        CooksDOLSlimit=4/size(LevX,1);
        
        %Figure for influence values
        figure
        OXIn=linspace(1,size(A,1),size(A,1))';
        OYIn=CooksDOLS(:,1);
        OCLim=(ones(size(OXLe,1),1)).*CooksDOLSlimit;
        plot(OXIn,OYIn,'ko',OXIn,OCLim,'r-')
        xlabel('Observation','FontName','times','FontSize',14)
        ylabel('Influence Value / Cooks D','FontName','times','FontSize',14)
        title('Influence Values versus Observations','FontName','times','FontSize',14);
        box('on')   %places border on all four sides
        grid('on')
        
        SSres=((YOLS-XOLS*ZOLS)')*(YOLS-XOLS*ZOLS);
        save SSres.txt SSres -ASCII
        SStot=(YOLS'*YOLS)-(1/size(YOLS,1))*(sum(YOLS(:,1)))^(2);
        save SStot.txt SStot -ASCII
            
            kecdabc=1;
            for kecdab=1:size(LevOLS,1)
                SizeLevOLS(kecdab,1)=kecdabc;
                SizeCooksDOLS(kecdab,1)=kecdabc;
                kecdabc=kecdabc+1;
            end
            COEFFINFO1=[FinalCoeff(:,1) CoeffStdErr(:,1) TCoeff(:,1) ProbTCoeff(:,1)];
            COEFFINFO2=[FinalCoeff(2:size(COEFFINFO1,1),1)'; CoeffStdErr(2:size(COEFFINFO1,1),1)'; TCoeff(2:size(COEFFINFO1,1),1)'; ProbTCoeff(2:size(COEFFINFO1,1),1)'];
            LevInf=[SizeLevOLS(:,1)'; COLSOrigRes(:,1)'; LevOLS(:,1)'; CooksDOLS(:,1)'];
            %COLSOrigRes2=mean(COLSOrigResSq(:,1));
            COLSOrigRes2=sum(COLSOrigResSq(:,1))/(size(XOLS,1)-(size(XOLS,2)-1)-1);
            SST=YOLS'*YOLS-((sum(YOLS(:,1))^2)/size(YOLS,1));
            SSR=FinalCoeff'*XOLS'*YOLS-((sum(YOLS(:,1))^2)/size(YOLS,1));
            OLSR2=100*(SSR/SST);
            %OrigAVP1=100*((exp(5.302*OrigAVP)-1)^(0.5));
            %ca2=100*((exp(5.302*ca)-1)^(0.5));
            
            fid=fopen('ConventionalOLS.txt','w');
            fprintf(fid, 'WREG v%s\n', versionnum); % print version number in header
            fprintf(fid, '%s\n \n', versiondate); % print version date
            fprintf(fid,'Regression Model for %s\n',Name_B_Selected);
            fprintf(fid,'                                                     \n');
            fprintf(fid,'Performance Metrics                                \n');
            fprintf(fid,'Mean Squared Error             %10.3f (log units)  \n',COLSOrigRes2);
            fprintf(fid,'R2                             %10.2f (percent)    \n',OLSR2);
            %fprintf(fid,'Effective Years of Record          NA              \n');
            fprintf(fid,'Standard Model Error               NA              \n');
            fprintf(fid,'                                                   \n');
            fprintf(fid,'Coefficients of Model                               \n');
            fprintf(fid,'                        Std Error  T value   P>|T|  \n');
            fprintf(fid,'Constant    %10.3f     %6.2f   %6.2f  %6.3f\n',COEFFINFO1(1,:));
%IC            fprintf(fid,'Coefficient %10.2f     %6.2f   %6.2f  %6.3f\n',COEFFINFO2);
            n_space=12-size(Name_A_Selected,2);
            text_space='';
            for ispace=1:n_space
                text_space=[text_space ' '];
            end
            for iprint=1:N_A_Selected
                fprintf(fid,'%s%s%10.3f     %6.2f   %6.2f  %6.3f\n',Name_A_Selected(iprint,:),text_space,COEFFINFO2(:,iprint));
            end
            fprintf(fid,'                                                   \n');
            fprintf(fid,'Transformations           C1      C2      C3      C4\n');
            n_space=12-size(Name_B_Selected,2);
            text_space='';
            for ispace=1:n_space
                text_space=[text_space ' '];
            end
            switch B_Transform
                case 1
                    Tran_text='       1';
                case 2
                    Tran_text='   log10';
                case 3
                    Tran_text='      ln';
                case 4
                    Tran_text='     exp';
            end
            fprintf(fid,'%s%s%s%8.2f%8.2f%8.2f%8.2f\n',Name_B_Selected(1,:),text_space,Tran_text,B_Constant(1,:));
            n_space=12-size(Name_A_Selected,2);
            text_space='';
            for ispace=1:n_space
                text_space=[text_space ' '];
            end
            for iprint=1:N_A_Selected
                switch A_Transform(iprint)
                    case 1
                        Tran_text='       1';
                    case 2
                        Tran_text='   log10';
                    case 3
                        Tran_text='      ln';
                    case 4
                        Tran_text='     exp';
                end
                fprintf(fid,'%s%s%s%8.2f%8.2f%8.2f%8.2f\n',Name_A_Selected(iprint,:),text_space,Tran_text,A_Constant(iprint,:));
            end
            fprintf(fid,'                                                   \n');
            fprintf(fid,'Residuals, Leverage, and Influence of Observations   \n');
            fprintf(fid,'Leverage Limit                 %10.3f          \n',ConvOLSHlimit);
            fprintf(fid,'Influence Limit                %10.3f          \n',CooksDOLSlimit);
            fprintf(fid,'             Observation  Residual Leverage Influence\n');
            fprintf(fid,'%23.0f%10.3f%10.3f%10.3f\n',LevInf(:,:));
            %fprintf(fid,'*Note: All 9999 indicate no logistic regession available. All -9999 indicate N/A. Regr.       \n');
            fclose(fid);
   
    end
    
else    %RegType==2, RoI regression    
    GRNum=linspace(1,size(SiteInfo.data,1),size(SiteInfo.data,1))';
    GR=[SiteInfo.data(:,7) GRNum];  %Regional Skew Values at each gage & Site no
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %EUCLIDEAN DISTANCE CALCULATION FOR ALL UNGAGED SITES PREDICTORS%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %Computation of Euclidean Distance among all sites (Measures proximity in predictor-variable space)
    [D]=EucDistance(A,B,C,E);     %Calling out subroutine 'EucDistance'
    %save D.out D -ASCII             %Saving Euclidean distance matrix as an ASCII file format
    SN=linspace(1,size(A,1),size(A,1))';    %Station no used
    D2=[D(:,:) zeros(size(D,1),2) SN(:,:) SIZELF2(:,:)];
    
    if ParFit==3 | ParFit==2    %Needs to be separate from iterative loop below
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %SIGMA REGRESSION FOR GLS REGRESSION%
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %GLOBAL Sigma Regression, so using global parameter values instead of localized RoI ones (Regressing by multiple linear regression between each stations std dev on annual peaks to corresponding basin attributes)
        GlobalPreds=[D(1:size(D,1), size(A,1)+1:size(A,1)+size(A,2)+size(B,2)+size(C,2)+size(E,2))];
        GXBasin=[ones(size(GlobalPreds,1),1) GlobalPreds(1:size(GlobalPreds,1), 1:size(A,2))];                  %Predictors at each selected station
        GLSigma=[GlobalPreds(1:size(GlobalPreds,1), size(A,2)+size(B,2)+size(C,2)+1)];                          %Lsigma values
        SigCoeff=regress(GLSigma,GXBasin);                                                                      %'GLSigma' is the log(sigma) matrix of values
    else
        
    end
    
    NStat=size(A,1);    %No. of gaged (treated as ungaged) stations=no. of regression equations
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %REGION-OF-INFLUENCE REGRESSION MODELS ARE FORMED%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    for c=1:NStat   %RoI regression-each gaged site is treated one at a time as ungaged
        ITERATION=c
        
        S=sortrows(D2,c); %Sorting for each ungaged location based on the Euclidean distance
        
        %Distance calculation from ungaged to all gaged sites in ENTIRE STUDY AREA
        for tt=1:size(A,1)
            if tt==1
                S(tt,(size(A,1)+size(A,2)+size(B,2)+size(C,2)+size(E,2)+1))=0;
            else    
                S(tt,(size(A,1)+size(A,2)+size(B,2)+size(C,2)+size(E,2)+1))=((abs(S(1, size(A,1)+size(A,2)+size(B,2)+1)-S(tt, size(A,1)+size(A,2)+size(B,2)+1))*1852*60)^(2)+(abs(S(1, size(A,1)+size(A,2)+size(B,2)+2)-S(tt, size(A,1)+size(A,2)+size(B,2)+2))*1852*60)^(2))^(0.5);
            end
        end
        
        %Counter to check when we have taken nmin(c,1) stations that are LESS THAN 'MaxR'
        Count=0;
        uu=1;
        vv=uu;
        for ww=(uu+1):size(A,1)
            if S(ww,(size(A,1)+size(A,2)+size(B,2)+size(C,2)+size(E,2)+1))<=MaxR & Count<nmin
                vv=ww;
                Count=Count+1;
            else
            
            end
        end
        if vv==(nmin+1)     %PRoI     %Count==nmin  %Case where I have enough sites less than MaxR and closest in predictor-variable space
            %S2=[S(2:vv+nmin, size(A,1)+1:size(A,1)+size(A,2)+size(B,2)+size(C,2)+size(E,2)+1)];
            S2=[S(2:vv, size(A,1)+1:size(A,1)+size(A,2)+size(B,2)+size(C,2)+size(E,2)+1+2+1)];
            S3=S2;
            %Final extraction of nmin stations that are <MaxR to the ungaged location
            ExtrS=[S3(1:nmin,1:size(A,2)+size(B,2)+size(C,2)+size(E,2)+1+2+1)];
        elseif vv>(nmin+1) & Count==nmin
            S2=[S(2:vv, size(A,1)+1:size(A,1)+size(A,2)+size(B,2)+size(C,2)+size(E,2)+1+2+1)];
            S3=sortrows(S2,size(A,2)+size(B,2)+size(C,2)+size(E,2)+1);
            ExtrS=[S3(1:nmin,1:size(A,2)+size(B,2)+size(C,2)+size(E,2)+1+2+1)];
        else    %GRoI   %Count<nmin %Case where I don't have enough sites less than MaxR, so take nmin geographically closest sites
            Distance=size(A,1)+size(A,2)+size(B,2)+size(C,2)+size(E,2)+1;
            S2=sortrows(S,Distance);
            S3=[S2(:,size(A,1)+1:size(A,1)+size(A,2)+size(B,2)+size(C,2)+size(E,2)+1+2+1)];
            ExtrS=[S2(2:nmin+1,size(A,1)+1:size(A,1)+size(A,2)+size(B,2)+size(C,2)+size(E,2)+1+2+1)];
        end
        %The 'ExtrS' matrixs contains the following info basin attributes, predictands, lat and long, at-site Lsigma, K, weighted skew for all the basins in the RoI
        
        XBasin=[ones(size(ExtrS,1),1) ExtrS(1:size(ExtrS,1), 1:size(A,2))]; %Predictors of all gauged basins in the RoI
        YBasin=[ExtrS(1:size(ExtrS,1), (size(A,2)+ChY))]; %Predictands of all gauged basins in the RoI
        
        if ParFit==3    %For GLS only
            %Calculating the sample Std Dev, si, at each gauged basin in the RoI (Using si as estimator of sigma i)
            for f=1:size(ExtrS,1)
                SumSigma=0;
                for fefa=1:size(A,2)
                    SumSigma=SumSigma+(SigCoeff(fefa+1,1)*XBasin(f,fefa+1));
                end
                Sigma(f,1)=SigCoeff(1,1)+SumSigma;
            end
            
            for s=1:size(ExtrS,1)
                ExtrS(s,size(A,2)+size(B,2)+size(C,2)+size(E,2)+2)=Sigma(s,1);
            end
        else
            
        end
        %The 'ExtrS' matrixs contains the following info basin attributes, predictands, lat and long, at-site Lsigma, K, weighted skew, estimates sigma values for all the basins in the RoI
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %DETERMINING THE WEIGHTING MATRIX TO USE IN A REGRESSION MODEL%
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        if ParFit==3    %GLS fitting procedure by Tasker and Stedinger (1989)
            
            if UnSkew==1
                [FinalCoeff(:,c), SelGSQ(c,1), LambdaF]=NewLambdaMatrixFinalRoI(A,B,C,E,Sigma,ExtrS,Theta,Alpha,ChY,Conm,RT,RegionSkew,Exceed,GR);    
            else
                [FinalCoeff(:,c), SelGSQ(c,1), LambdaF]=LambdaMatrixFinalRoI(A,B,C,E,Sigma,ExtrS,Theta,Alpha,ChY,Conm);   %Calling out subroutine 'LambdaMatrix'
                %The output from this subroutine are the final coefficients, the model error variance by method of moments, and the final Lambda weighting matrix of cross correlation among basins
            end
                
            %New RoI specific performance metrics for leverage and influence
            [Indsecterm, LevRoI, ProbPts2, NoLGLEVi, HLev, INFPts2, NoINFi, HLeverageLimit]=RoIMetrics(ExtrS,A,LambdaF,S,FinalCoeff(:,c),ChY);    %Calling out subroutine 'RoIMetrics'
            %Number of large leverage and influential observations
            LevRoI1(c,:)=LevRoI;
            NoLGLEV(c,1)=NoLGLEVi;    
            NoINF(c,1)=NoINFi;
            GagesUsed(c,:)=ExtrS(:,size(A,2)+size(B,2)+size(C,2)+size(E,2)+1+2)';
            
            %save HLeverageLimit.out HLeverageLimit -ASCII
            %save LevRoI1.out LevRoI1 -ASCII
            
        elseif ParFit==2    %WLS fitting procedure by Tasker (1980)
            %Calculating the sample Std Dev, si, at each gauged basin in the RoI (Using si as estimator of sigma i)
            for fff=1:size(ExtrS,1)
                SumSigma=0;
                for fefb=1:size(A,2)
                    SumSigma=SumSigma+(SigCoeff(fefb+1,1)*XBasin(fff,fefb+1));
                end
                Sigma(fff,1)=SigCoeff(1,1)+SumSigma;
            end
            
            for sss=1:size(ExtrS,1)
                ExtrS(sss,size(A,2)+size(B,2)+size(C,2)+size(E,2)+2)=Sigma(sss,1);
            end
            
            %Calculating weighting function (wWLS=1/(ca+cb*(1/n)))
            AvgSigma=mean(Sigma(:,1));
            AvgK=mean(ExtrS(:,size(A,2)+size(B,2)+size(C,2)+2));
            Avgg=mean(ExtrS(:,size(A,2)+size(B,2)+size(C,2)+3));
            cb=(AvgSigma^2)*(1+(((AvgK^2)/2)*(1+(0.75*(Avgg^2))))+(AvgK*Avgg));
            if cb<0
                cb=0;
            else
                
            end
            %Calculating the observed standard error of estimate using OLS
            XOLS2=[ones(size(ExtrS,1),1) ExtrS(1:size(ExtrS,1), 1:size(A,2))];
            YOLS2=[ExtrS(1:size(ExtrS,1), size(A,2)+ChY)];
            ZOLS2=regress(YOLS2,XOLS2); 
            FCoeff(:,c)=ZOLS2;
            
            for ttt=1:size(XOLS2,1)
                SumOLS2=0;
                for bbba=1:size(A,2)
                    SumOLS2=SumOLS2+(FCoeff(bbba+1,c)*XOLS2(ttt,bbba+1));
                end
                YESTOLS2(ttt,c)=FCoeff(1,c)+SumOLS2;
                ResOLS2(ttt,c)=YOLS2(ttt,1)-YESTOLS2(ttt,c);
            end
            
            ObsVar(c,1)=(std(ResOLS2(:,c)))^2;
            %Calculating Avg Inverse Record Lengths from POR, mWLS
            %Calculating Record Lengths from POR, m
            for ffff=1:size(SIZELF2,1)    
                m(ffff,1)=SIZELF2(ffff,1);
                mWLS(ffff,1)=(m(ffff,1))^-1;
            end
            for hhhh=1:size(ExtrS,1)
                ExtrS(hhhh,1+size(A,2)+size(B,2)+size(C,2)+size(E,2)+1+size(Sigma,2)+1)=mWLS(hhhh,1);
            end
            
            Avgn=mean(mWLS(:,1));
            
            ca=ObsVar(c,1)-(cb*Avgn);
            if ca<0
                ca=0;
            else
                
            end
            for www=1:size(ExtrS,1)   
                wWLS(www,c)=(ca+cb*(mWLS(www,1)));   %A vector of the weights for each c iteration.
            end    
            Ident=eye(size(wWLS,1));
            for xxx=1:size(Ident,1)
                for yyy=1:size(Ident,1)
                    if xxx==yyy
                        Wmatrix(xxx,yyy)=wWLS(xxx,c);
                    else
                        
                    end
                end
            end
            ZWLS=inv(XBasin'*inv(Wmatrix)*XBasin)*XBasin'*inv(Wmatrix)*YBasin;  
            FinalCoeff(:,c)=ZWLS;          
            GagesUsed(c,:)=ExtrS(:,size(A,2)+size(B,2)+size(C,2)+size(E,2)+1+2)';
  
            RWLSMultLev=4;  %Multiplier to Leverage Limit to apply (in conventional regression we use a value of 2).
            RWLSEstBC=[1 S(1, (size(A,1)+1):(size(A,1)+size(A,2)))];
            RWLSX=[ones(size(ExtrS,1),1) ExtrS(1:size(ExtrS,1), 1:size(A,2))];
            RWLSY=[ExtrS(1:size(ExtrS,1), size(A,2)+ChY)];
            RWLSLevRoI=RWLSEstBC*inv(RWLSX'*inv(Wmatrix)*RWLSX)*RWLSX'*inv(Wmatrix);  %Newly defined RoI hat vector of leverage values.
            WHLeverageLimit=(RWLSMultLev*sum(RWLSLevRoI(1,1:size(ExtrS,1))))/size(ExtrS,1);
            RWLSLevRoI1(c,:)=RWLSLevRoI;
            
            %save WHLeverageLimit.out WHLeverageLimit -ASCII
            %save RWLSLevRoI1.out RWLSLevRoI1 -ASCII
            
        else    %ParFit==1  %Standard OLS fitting procedure
            XOLS=[ones(size(ExtrS,1),1) ExtrS(1:size(ExtrS,1), 1:size(A,2))];
            YOLS=[ExtrS(1:size(ExtrS,1), size(A,2)+ChY)];
            ZOLS=regress(YOLS,XOLS); 
            FinalCoeff(:,c)=ZOLS;
            GagesUsed(c,:)=ExtrS(:,size(A,2)+size(B,2)+size(C,2)+size(E,2)+1+2)';
            
            ROLSMultLev=4;  %Multiplier to Leverage Limit to apply (in conventional regression we use a value of 2).
            ROLSEstBC=[1 S(1, (size(A,1)+1):(size(A,1)+size(A,2)))];
            ROLSX=[ones(size(ExtrS,1),1) ExtrS(1:size(ExtrS,1), 1:size(A,2))];
            ROLSY=[ExtrS(1:size(ExtrS,1), size(A,2)+ChY)];
            ROLSLevRoI=ROLSEstBC*inv(ROLSX'*ROLSX)*ROLSX';  %Newly defined RoI hat vector of leverage values.   
            OHLeverageLimit=(ROLSMultLev*sum(ROLSLevRoI(1,:)))/size(ExtrS,1);
            ROLSLevRoI1(c,:)=ROLSLevRoI;
            
            %save OHLeverageLimit.out OHLeverageLimit -ASCII
            %save ROLSLevRoI1.out ROLSLevRoI1 -ASCII
            
        end     %Associated with if loop 'ParFit'
        
        %%New RoI specific performance metrics for leverage and influence
        %[Indsecterm, LevRoI, ProbPts2, NoLGLEVi, HLev, INFPts2, NoINFi, HLeverageLimit]=RoIMetrics(ExtrS,A,LambdaF,S,FinalCoeff(:,c));    %Calling out subroutine 'RoIMetrics'
        SumYEST=0;    
        for abcd=1:size(A,2)
            SumYEST=SumYEST+(FinalCoeff(abcd+1,c)*S(1, size(A,1)+abcd));
        end
        OrigYEST(c,1)=FinalCoeff(1,c)+SumYEST; %Estimated YHat
        OrigRes(c,1)=S(1, size(A,1)+size(A,2)+ChY)-OrigYEST(c,1);     %Unscaled residual
        OrigResSq(c,1)=OrigRes(c,1)^2;                              %Unscaled squared residual
        MOrigResSq=mean(OrigResSq(:,1));
    end     %Associated with the 'c' loop
    
    if ParFit==3
            MOrigResSq2=100*((exp(5.302*MOrigResSq)-1)^(0.5));
            
            kecdabc=1;
            for kecdab=1:size(GagesUsed,1)
                RegrNo(kecdab,1)=kecdabc;
                kecdabc=kecdabc+1;
            end
            
            %KE(5-30-08)Addition of reporting PRESS_RoI residuals
            PRROI=[RegrNo(:,1)'; OrigResSq(:,1)'];
            
            fid=fopen('RegionofInfluenceGLS.txt','w');
            fprintf(fid, 'WREG v%s \n', versionnum); % print version number in header
            fprintf(fid, '%s \n \n', versiondate); % print version date
            fprintf(fid,'Region-of-Influence Regression Models for %s\n',Name_B_Selected);
            fprintf(fid,'                                                     \n');
            fprintf(fid,'Performance Metrics                                \n');
            fprintf(fid,'Root Mean Square Error         %10.2f          \n',MOrigResSq2);
            fprintf(fid,'Pseudo R2                          NA              \n');
            %fprintf(fid,'Effective Years of Record          NA              \n');
            fprintf(fid,'Standard Model Error               NA              \n');
            fprintf(fid,'*Note: Root Mean Square Error is in percent.       \n');
            fprintf(fid,'                                                   \n');
            fprintf(fid,'Coefficients of Model                              \n');
            fprintf(fid,'Regression    Constant    Coefficients                 \n');
            fprintf(fid,'                      ');
            Name_A_Space=double(Name_A_Selected);
            for iprint=1:N_A_Selected
                flag=0;
                for ispace=1:size(Name_A_Space,2)
                    if Name_A_Space(iprint,ispace)==32   %IC look for filling space at the end of name
                        flag=1;
                        break;
                    end
                end
                if flag==0   %IC no filling space
                    ispace=ispace+1;
                end  %IC the length of the string is ispace-1
                n_space=12-(ispace-1);
                for print_space=1:n_space
                    fprintf(fid,' ');
                end
                fprintf(fid,'%s',Name_A_Selected(iprint,1:ispace-1));
            end
            fprintf(fid,'\n');
            for ke21a=1:size(GagesUsed,1)
                fprintf(fid,'%10.0f',RegrNo(ke21a,1));
                for ke21b=1:size(FinalCoeff,1)
                    if ke21b==size(FinalCoeff,1)
                        fprintf(fid,'%12.3f\n',FinalCoeff(ke21b,ke21a));
                    else
                        fprintf(fid,'%12.3f',FinalCoeff(ke21b,ke21a));
                    end
                end
            end
            fprintf(fid,'                                                   \n');
            fprintf(fid,'Transformations           C1      C2      C3      C4\n');
            n_space=12-size(Name_B_Selected,2);
            text_space='';
            for ispace=1:n_space
                text_space=[text_space ' '];
            end
            switch B_Transform
                case 1
                    Tran_text='       1';
                case 2
                    Tran_text='   log10';
                case 3
                    Tran_text='      ln';
                case 4
                    Tran_text='     exp';
            end
            fprintf(fid,'%s%s%s%8.2f%8.2f%8.2f%8.2f\n',Name_B_Selected(1,:),text_space,Tran_text,B_Constant(1,:));
            n_space=12-size(Name_A_Selected,2);
            text_space='';
            for ispace=1:n_space
                text_space=[text_space ' '];
            end
            for iprint=1:N_A_Selected
                switch A_Transform(iprint)
                    case 1
                        Tran_text='       1';
                    case 2
                        Tran_text='   log10';
                    case 3
                        Tran_text='      ln';
                    case 4
                        Tran_text='     exp';
                end
                fprintf(fid,'%s%s%s%8.2f%8.2f%8.2f%8.2f\n',Name_A_Selected(iprint,:),text_space,Tran_text,A_Constant(iprint,:));
            end
            fprintf(fid,'                                                   \n');
            fprintf(fid,'PRESS_RoI Residuals                                \n');
            fprintf(fid,'Observation                 Residual               \n');
            fprintf(fid,'%8.0f%28.3f\n',PRROI(:,:));
            fprintf(fid,'                                                   \n');
            fprintf(fid,'Leverage and Influence                             \n');
            fprintf(fid,'Leverage Limit                 %10.3f          \n',HLeverageLimit);
            fprintf(fid,'Influence Limit                        NA          \n');
            fprintf(fid,'Regression             Observations Included in RoI\n');
            fprintf(fid,'  Model                     and Leverage Value     \n');
            for ke21c=1:size(GagesUsed,1)
                fprintf(fid,'%10.0f',RegrNo(ke21c,1));
                for ke21d=1:size(LevRoI1,2)
                    if ke21d==size(LevRoI1,2)
                        fprintf(fid,'%6.0f%7.3f\n',GagesUsed(ke21c,ke21d),LevRoI1(ke21c,ke21d));
                    else
                        fprintf(fid,'%6.0f%7.3f',GagesUsed(ke21c,ke21d),LevRoI1(ke21c,ke21d));
                    end
                end
            end
            fclose(fid);

        elseif ParFit==2
            MOrigResSq2=100*((exp(5.302*MOrigResSq)-1)^(0.5));
            
            kecdabc=1;
            for kecdab=1:size(GagesUsed,1)
                RegrNo(kecdab,1)=kecdabc;
                kecdabc=kecdabc+1;
            end
            
            %KE(5-30-08)Addition of reporting PRESS_RoI residuals
            PRROI=[RegrNo(:,1)'; OrigResSq(:,1)'];
            
            fid=fopen('RegionofInfluenceWLS.txt','w');
            fprintf(fid, 'WREG v%s \n', versionnum); % print version number in header
            fprintf(fid, '%s \n \n', versiondate); % print version date
            fprintf(fid,'Region-of-Influence Regression Models for %s\n',Name_B_Selected);
            fprintf(fid,'                                                     \n');
            fprintf(fid,'Performance Metrics                                \n');
            fprintf(fid,'Root Mean Square Error         %10.2f          \n',MOrigResSq2);
            fprintf(fid,'Pseudo R2                          NA              \n');
            %fprintf(fid,'Effective Years of Record          NA              \n');
            fprintf(fid,'Standard Model Error               NA              \n');
            fprintf(fid,'*Note: Root Mean Square Error is in percent.       \n');
            fprintf(fid,'                                                   \n');
            fprintf(fid,'Coefficients of Model                              \n');
            fprintf(fid,'Regression    Constant    Coefficients                 \n');
            fprintf(fid,'                      ');
            Name_A_Space=double(Name_A_Selected);
            for iprint=1:N_A_Selected
                flag=0;
                for ispace=1:size(Name_A_Space,2)
                    if Name_A_Space(iprint,ispace)==32   %IC look for filling space at the end of name
                        flag=1;
                        break;
                    end
                end
                if flag==0   %IC no filling space
                    ispace=ispace+1;
                end  %IC the length of the string is ispace-1
                n_space=12-(ispace-1);
                for print_space=1:n_space
                    fprintf(fid,' ');
                end
                fprintf(fid,'%s',Name_A_Selected(iprint,1:ispace-1));
            end
            fprintf(fid,'\n');
            for ke21a=1:size(GagesUsed,1)
                fprintf(fid,'%10.0f',RegrNo(ke21a,1));
                for ke21b=1:size(FinalCoeff,1)
                    if ke21b==size(FinalCoeff,1)
                        fprintf(fid,'%12.3f\n',FinalCoeff(ke21b,ke21a));
                    else
                        fprintf(fid,'%12.3f',FinalCoeff(ke21b,ke21a));
                    end
                end
            end
            fprintf(fid,'                                                   \n');
            fprintf(fid,'Transformations           C1      C2      C3      C4\n');
            n_space=12-size(Name_B_Selected,2);
            text_space='';
            for ispace=1:n_space
                text_space=[text_space ' '];
            end
            switch B_Transform
                case 1
                    Tran_text='       1';
                case 2
                    Tran_text='   log10';
                case 3
                    Tran_text='      ln';
                case 4
                    Tran_text='     exp';
            end
            fprintf(fid,'%s%s%s%8.2f%8.2f%8.2f%8.2f\n',Name_B_Selected(1,:),text_space,Tran_text,B_Constant(1,:));
            n_space=12-size(Name_A_Selected,2);
            text_space='';
            for ispace=1:n_space
                text_space=[text_space ' '];
            end
            for iprint=1:N_A_Selected
                switch A_Transform(iprint)
                    case 1
                        Tran_text='       1';
                    case 2
                        Tran_text='   log10';
                    case 3
                        Tran_text='      ln';
                    case 4
                        Tran_text='     exp';
                end
                fprintf(fid,'%s%s%s%8.2f%8.2f%8.2f%8.2f\n',Name_A_Selected(iprint,:),text_space,Tran_text,A_Constant(iprint,:));
            end
            fprintf(fid,'                                                   \n');
            fprintf(fid,'PRESS_RoI Residuals                                \n');
            fprintf(fid,'Observation                 Residual               \n');
            fprintf(fid,'%8.0f%28.3f\n',PRROI(:,:));
            fprintf(fid,'                                                   \n');
            fprintf(fid,'Leverage and Influence                             \n');
            fprintf(fid,'Leverage Limit                 %10.3f          \n',WHLeverageLimit);
            fprintf(fid,'Influence Limit                        NA          \n');
            fprintf(fid,'Regression             Observations Included in RoI\n');
            fprintf(fid,'  Model                     and Leverage Value     \n');
            for ke21c=1:size(GagesUsed,1)
                fprintf(fid,'%10.0f',RegrNo(ke21c,1));
                for ke21d=1:size(RWLSLevRoI1,2)
                    if ke21d==size(RWLSLevRoI1,2)
                        fprintf(fid,'%6.0f%7.3f\n',GagesUsed(ke21c,ke21d),RWLSLevRoI1(ke21c,ke21d));
                    else
                        fprintf(fid,'%6.0f%7.3f',GagesUsed(ke21c,ke21d),RWLSLevRoI1(ke21c,ke21d));
                    end
                end
            end
            fclose(fid);
            
        else
            MOrigResSq2=100*((exp(5.302*MOrigResSq)-1)^(0.5));
            
            kecdabc=1;
            for kecdab=1:size(GagesUsed,1)
                RegrNo(kecdab,1)=kecdabc;
                kecdabc=kecdabc+1;
            end
            
            %KE(5-30-08)Addition of reporting PRESS_RoI residuals
            PRROI=[RegrNo(:,1)'; OrigResSq(:,1)'];
            
            fid=fopen('RegionofInfluenceOLS.txt','w');
            fprintf(fid, 'WREG v%s \n', versionnum); % print version number in header
            fprintf(fid, '%s \n \n', versiondate); % print version date
            fprintf(fid,'Region-of-Influence Regression Models for %s\n',Name_B_Selected);
            fprintf(fid,'                                                     \n');
            fprintf(fid,'Performance Metrics                                \n');
            fprintf(fid,'Root Mean Square Error         %10.2f          \n',MOrigResSq2);
            fprintf(fid,'Pseudo R2                          NA              \n');
            %fprintf(fid,'Effective Years of Record          NA              \n');
            fprintf(fid,'Standard Model Error               NA              \n');
            fprintf(fid,'*Note: Root Mean Square Error is in percent.       \n');
            fprintf(fid,'                                                   \n');
            fprintf(fid,'Coefficients of Model                              \n');
            fprintf(fid,'Regression    Constant    Coefficients                 \n');
            fprintf(fid,'                      ');
            Name_A_Space=double(Name_A_Selected);
            for iprint=1:N_A_Selected
                flag=0;
                for ispace=1:size(Name_A_Space,2)
                    if Name_A_Space(iprint,ispace)==32   %IC look for filling space at the end of name
                        flag=1;
                        break;
                    end
                end
                if flag==0   %IC no filling space
                    ispace=ispace+1;
                end  %IC the length of the string is ispace-1
                n_space=12-(ispace-1);
                for print_space=1:n_space
                    fprintf(fid,' ');
                end
                fprintf(fid,'%s',Name_A_Selected(iprint,1:ispace-1));
            end
            fprintf(fid,'\n');
            for ke21a=1:size(GagesUsed,1)
                fprintf(fid,'%10.0f',RegrNo(ke21a,1));
                for ke21b=1:size(FinalCoeff,1)
                    if ke21b==size(FinalCoeff,1)
                        fprintf(fid,'%12.3f\n',FinalCoeff(ke21b,ke21a));
                    else
                        fprintf(fid,'%12.3f',FinalCoeff(ke21b,ke21a));
                    end
                end
            end
            fprintf(fid,'                                                   \n');
            fprintf(fid,'Transformations           C1      C2      C3      C4\n');
            n_space=12-size(Name_B_Selected,2);
            text_space='';
            for ispace=1:n_space
                text_space=[text_space ' '];
            end
            switch B_Transform
                case 1
                    Tran_text='       1';
                case 2
                    Tran_text='   log10';
                case 3
                    Tran_text='      ln';
                case 4
                    Tran_text='     exp';
            end
            fprintf(fid,'%s%s%s%8.2f%8.2f%8.2f%8.2f\n',Name_B_Selected(1,:),text_space,Tran_text,B_Constant(1,:));
            n_space=12-size(Name_A_Selected,2);
            text_space='';
            for ispace=1:n_space
                text_space=[text_space ' '];
            end
            for iprint=1:N_A_Selected
                switch A_Transform(iprint)
                    case 1
                        Tran_text='       1';
                    case 2
                        Tran_text='   log10';
                    case 3
                        Tran_text='      ln';
                    case 4
                        Tran_text='     exp';
                end
                fprintf(fid,'%s%s%s%8.2f%8.2f%8.2f%8.2f\n',Name_A_Selected(iprint,:),text_space,Tran_text,A_Constant(iprint,:));
            end
            fprintf(fid,'                                                   \n');
            fprintf(fid,'PRESS_RoI Residuals                                \n');
            fprintf(fid,'Observation                 Residual               \n');
            fprintf(fid,'%8.0f%28.3f\n',PRROI(:,:));
            fprintf(fid,'                                                   \n');
            fprintf(fid,'Leverage and Influence                             \n');
            fprintf(fid,'Leverage Limit                 %10.3f          \n',OHLeverageLimit);
            fprintf(fid,'Influence Limit                        NA          \n');
            fprintf(fid,'Regression             Observations Included in RoI\n');
            fprintf(fid,'  Model                     and Leverage Value     \n');
            for ke21c=1:size(GagesUsed,1)
                fprintf(fid,'%10.0f',RegrNo(ke21c,1));
                for ke21d=1:size(ROLSLevRoI1,2)
                    if ke21d==size(ROLSLevRoI1,2)
                        fprintf(fid,'%6.0f%7.3f\n',GagesUsed(ke21c,ke21d),ROLSLevRoI1(ke21c,ke21d));
                    else
                        fprintf(fid,'%6.0f%7.3f',GagesUsed(ke21c,ke21d),ROLSLevRoI1(ke21c,ke21d));
                    end
                end
            end
            fclose(fid);
            
        end
    
    %save FinalCoeff.out FinalCoeff -ASCII
    %save OrigRes.out OrigRes -ASCII
    %save GagesUsed.out GagesUsed -ASCII
    
end    %Associated with if loop 'RegType'


if RegType==1
    fid=fopen('RegressionModel.txt','w');

%IC  Form regression model
    C1=num2str(B_Constant(1),'%8.2f');
    C2=num2str(B_Constant(2),'%8.2f');
    C3=num2str(B_Constant(3),'%8.2f');
    C4=num2str(B_Constant(4),'%8.2f');
    Name_B_Space=double(Name_B_Selected);
    flag=0;
    for ispace=1:size(Name_B_Space,2)
        if Name_B_Space(ispace)==32   %IC look for filling space at the end of name
            flag=1;
            break;
        end
    end
    if flag==0   %IC no filling space
        ispace=ispace+1;
    end
    Var=Name_B_Selected(1,1:ispace-1);
    Coeff=num2str(COEFFINFO1(1,1),'%10.3f');
    switch B_Transform
        case 1
            RM_String=['[',C1,'*(',Var,')^(',C2,')+',C3,']^(',C4,')',' =',Coeff];
        case 2
            RM_String=['log_10','{[',C1,'*(',Var,')^(',C2,')+',C3,']^(',C4,')}',' =',Coeff];
        case 3
            RM_String=['ln','{[',C1,'*(',Var,')^(',C2,')+',C3,']^(',C4,')}',' =',Coeff];
        case 4
            RM_String=['e^','{[',C1,'*(',Var,')^(',C2,')+',C3,']^(',C4,')}',' =',Coeff];
    end

    for count_A=1:N_A_Selected
        C1=num2str(A_Constant(count_A,1),'%8.2f');
        C2=num2str(A_Constant(count_A,2),'%8.2f');
        C3=num2str(A_Constant(count_A,3),'%8.2f');
        C4=num2str(A_Constant(count_A,4),'%8.2f');
        Name_A_Space=double(Name_A_Selected);
        flag=0;
        for ispace=1:size(Name_A_Space,2)
            if Name_A_Space(count_A,ispace)==32   %IC look for filling space at the end of name
                flag=1;
                break;
            end
        end
        if flag==0   %IC no filling space
            ispace=ispace+1;
        end
        Var=Name_A_Selected(count_A,1:ispace-1);
        Coeff=num2str(COEFFINFO2(1,count_A),'%10.3f');
        if COEFFINFO2(1,count_A)>=0
            Coeff=['+',Coeff];
        end
        switch A_Transform(1,count_A)
            case 1
                RM_String=[RM_String,' ',Coeff,'*[',C1,'*(',Var,')^(',C2,')+',C3,']^(',C4,')'];
            case 2
                RM_String=[RM_String,' ',Coeff,'*log_10','{[',C1,'*(',Var,')^(',C2,')+',C3,']^(',C4,')}'];
            case 3
                RM_String=[RM_String,' ',Coeff,'*ln','{[',C1,'*(',Var,')^(',C2,')+',C3,']^(',C4,')}'];
            case 4
                RM_String=[RM_String,' ',Coeff,'*e^','{[',C1,'*(',Var,')^(',C2,')+',C3,']^(',C4,')}'];
        end 
    end
    fprintf(fid,'%s\n',RM_String);
    
    % added fclose JEK 2.9.2012
    fclose(fid);
end


if IGUI_Summary==1  %IC (Summary) GUI enabled
    pause(1);
    GUI_Summary;
end  %IC end (Summary) GUI


%IC  Record entries in Command Window to EventLog.txt
diary('off');
