function[FinalCoeff,SelGSQ,LambdaF]=NewLambdaMatrixFinalagvPRL(A,B,C,E,Sigma,m,dgeo,ExtrS,Theta,Alpha,ChY,Conm,RT,RegionSkew,Exceed,GR,PRL);
%Generalized least squares (GLS) lambda weighting matrix for streamflow characteristics.

%'Sigma'-The sigma regression on each site's std dev of annual peak data vs basin attributes (performed on entire dataset) is in main program. The estimated std dev are in the 'Sigma' matrix.
%'ExtrS'-is a matrix holding the list of sites to use region-of-influence regression and associated basin attributes, std dev of annual peaks, POR, Lat, and Long.
%'A'-is the input matrix containing the predictors.
%'B'-is a matrix containing the predictands.
%'C'-is a matrix containing the Lat and Long.
%'E'-is a matrix containing the Lsigma, K, & g values(local station values of skew).
%'F'-is a matrix containing the POR.
%'ChY'-is the column no. in the YHat data input matrix to use as the predictand.

%NEW INPUTS REQUIRED
%'RT'=single value or vector (one for each column in B) for 1/(return period of interest) specified by user
%'RegionSkew'=regionally estimated skew values for each site, such as using Bayesian skew estimation or WLS by Gary and Jery
%'Exceed'=whether looking at exceedence probability (peaks) or nonexceedence (low-flows)
%RT=0.01; %10 year return period
%RegionSkew=0.1*(ones(42,1)); %Each site will have an indiviudal estimate from the regional equation.
Nu=3; %needed for cross correlation function for at site skews (Martins and Stedinger, 2002, WRR)

for hh=1:size(ExtrS,1)
    ExtrS(hh,1+size(A,2)+size(B,2)+size(C,2)+size(E,2)+1+size(Sigma,2))=m(hh,1);
end

%SAMPLE CROSS CORRELATION MATRIX, 'rij'. Empirical Cross Correlation Function Fit to All Stations that have a Specified No. Yrs of Concurrent Record (TO BE ADDED).
%Using the constants derived from another state (North Carolina, 50 Yrs of concurrent record based on ~1150 stations that had this)
for c=1:size(ExtrS,1)
    for d=1:size(ExtrS,1)
        if c==d
            rij(d,c)=1;
        else 
            rij(d,c)=Theta^(((dgeo(d,c)/1000)*0.6214)/((Alpha*((dgeo(d,c)/1000)*0.6214))+1));   %Emprirical relationship between geographic distances and correlations among annual peak series
        end
    end
end




%PARTIAL K/PARTIAL STATION SKEW (PKPS) DERIVATIVE TERM (Griffis and Stedinger, 2007)
for aaa=1:size(E,1)
    if Exceed==1    
        %Nonexceedences (low-flows)
        PKPS(aaa,1)=((((norminv(RT))^2)-1)/6)+((1/3)*(((norminv(RT))^3)-(6*norminv(RT)))*(2/36)*(E(aaa,3)))-((((norminv(RT))^2)-1)*(3/(6^3))*(E(aaa,3)^2))+(norminv(RT)*(4/(6^4))*(E(aaa,3)^3))+((1/3)*(5/(6^5))*(E(aaa,3)^4));
    else
        %Exceedences (peak-flows)
        PKPS(aaa,1)=((((-norminv(RT))^2)-1)/6)+((1/3)*(((-norminv(RT))^3)-(6*-norminv(RT)))*(2/36)*(E(aaa,3)))-((((-norminv(RT))^2)-1)*(3/(6^3))*(E(aaa,3)^2))+(-norminv(RT)*(4/(6^4))*(E(aaa,3)^3))+((1/3)*(5/(6^5))*(E(aaa,3)^4));
    end 
end

%WEIGHT, W_i
%matrix 'm' contains the no. of years of record
%MSERegionSkew=0;
MSERegionSkew=RegionSkew;
%for bbb=1:size(E,1)
%    MSEg(bbb,1)=((6/m(bbb,1))+((-17.75/(m(bbb,1)^2))+(50.06/(m(bbb,1)^3))))*(1+(E(bbb,3)^2)*((9/6)+((3.93/(m(bbb,1)^0.3))-(30.97/(m(bbb,1)^0.6))+(37.1/(m(bbb,1)^0.9))))+(E(bbb,3)^4)*((15/48)+((-6.16/(m(bbb,1)^0.56))+(36.83/(m(bbb,1)^1.12))-(66.9/(m(bbb,1)^1.68)))));
%    Wi(bbb,1)=MSERegionSkew/(MSEg(bbb,1)+MSERegionSkew);
%end

%-----------------------------------------
%added by agv 8/6/15 for PRL fix
%PRL contains pseudo record lengths for each station for selected AEP.
for bbb=1:size(E,1)
    MSEg(bbb,1)=((6/PRL(bbb,1))+((-17.75/(PRL(bbb,1)^2))+(50.06/(PRL(bbb,1)^3))))*(1+(E(bbb,3)^2)*((9/6)+((3.93/(PRL(bbb,1)^0.3))-(30.97/(PRL(bbb,1)^0.6))+(37.1/(PRL(bbb,1)^0.9))))+(E(bbb,3)^4)*((15/48)+((-6.16/(PRL(bbb,1)^0.56))+(36.83/(PRL(bbb,1)^1.12))-(66.9/(PRL(bbb,1)^1.68)))));
    Wi(bbb,1)=MSERegionSkew/(MSEg(bbb,1)+MSERegionSkew);
end
%---------------------------------------


%COVARIANCE OF AT-SITE SKEW VALUES, 'COVgg'
%Computing the variance of at site skews
%for eee=1:size(E,1)
%    Vgi(eee,1)=((6/m(eee,1))+((-17.75/(m(eee,1)^2))+(50.06/(m(eee,1)^3))))*(1+(GR(eee,1)^2)*((9/6)+((3.92/(m(eee,1)^0.3))-(31.1/(m(eee,1)^0.6))+(34.86/(m(eee,1)^0.9))))+(GR(eee,1)^4)*((15/48)+((-7.31/(m(eee,1)^0.59))+(45.9/(m(eee,1)^1.18))-(86.5/(m(eee,1)^1.77)))));
%end
%for ccc=1:size(E,1)
%    for ddd=1:size(E,1)
%        if ccc==ddd
%            COVgg(ccc,ddd)=0;
%        else
%            if rij(ccc,ddd)>0
%                COVgg(ccc,ddd)=(1)*(Conm(ccc,ddd)/(((Conm(ccc,ddd)+m(ccc,1))*(Conm(ccc,ddd)+m(ddd,1)))^(0.5)))*((abs(rij(ccc,ddd)))^(Nu))*(Vgi(ccc,1)*Vgi(ddd,1))^(0.5);
%            else    %rij(ccc,ddd)<0
%                COVgg(ccc,ddd)=(-1)*(Conm(ccc,ddd)/(((Conm(ccc,ddd)+m(ccc,1))*(Conm(ccc,ddd)+m(ddd,1)))^(0.5)))*((abs(rij(ccc,ddd)))^(Nu))*(Vgi(ccc,1)*Vgi(ddd,1))^(0.5);
%            end
%        end
%    end
%end
%-----------------------------------------
%added by agv 8/6/15 for PRL fix
%PRL contains pseudo record lengths for each station for selected AEP.
for eee=1:size(E,1)
    Vgi(eee,1)=((6/PRL(eee,1))+((-17.75/(PRL(eee,1)^2))+(50.06/(PRL(eee,1)^3))))*(1+(GR(eee,1)^2)*((9/6)+((3.92/(PRL(eee,1)^0.3))-(31.1/(PRL(eee,1)^0.6))+(34.86/(PRL(eee,1)^0.9))))+(GR(eee,1)^4)*((15/48)+((-7.31/(PRL(eee,1)^0.59))+(45.9/(PRL(eee,1)^1.18))-(86.5/(PRL(eee,1)^1.77)))));
end
for ccc=1:size(E,1)
    for ddd=1:size(E,1)
        if ccc==ddd
            COVgg(ccc,ddd)=0;
        else
            if rij(ccc,ddd)>0
                COVgg(ccc,ddd)=(1)*(Conm(ccc,ddd)/(((Conm(ccc,ddd)+PRL(ccc,1))*(Conm(ccc,ddd)+PRL(ddd,1)))^(0.5)))*((abs(rij(ccc,ddd)))^(Nu))*(Vgi(ccc,1)*Vgi(ddd,1))^(0.5);
            else    %rij(ccc,ddd)<0
                COVgg(ccc,ddd)=(-1)*(Conm(ccc,ddd)/(((Conm(ccc,ddd)+PRL(ccc,1))*(Conm(ccc,ddd)+PRL(ddd,1)))^(0.5)))*((abs(rij(ccc,ddd)))^(Nu))*(Vgi(ccc,1)*Vgi(ddd,1))^(0.5);
            end
        end
    end
end
%---------------------------------------

%LAMBDA WEIGHTING MATRIX, Lambda. Assembling all the information in the previous subroutines to calculate Lambda
%An initial value of 'GammaSq' (ie model error) needs to be provided to start the iterative search for optimal b values. (Original method was the SECANT method from the Numerical Recipes book)
GammaSq1=0;
AnsChY=ChY
size(A,2)
save ExtrS.out ExtrS -ASCII
GammaSq2=2*(var(ExtrS(1:size(ExtrS,1),size(A,2)+ChY))); 
GInt=30;    
GSep=linspace(GammaSq1,GammaSq2,GInt)';
GSQ=[GSep(:,1) ones(size(GSep,1),2)]
for i=1:size(GSep,1)
    for j=1:size(ExtrS,1)
        for k=1:size(ExtrS,1)
            if j==k
                AddLT1M(k,j)=Wi(j,1)*ExtrS(j,size(A,2)+size(B,2)+size(C,2)+2)*PKPS(j,1)*(3*ExtrS(j,size(A,2)+size(B,2)+size(C,2)+3)+0.75*(ExtrS(j,size(A,2)+size(B,2)+size(C,2)+3))^(3));
                AddLT2M(k,j)=(Wi(j,1)^2)*(PKPS(j,1)^2)*(6+9*(ExtrS(j,size(A,2)+size(B,2)+size(C,2)+3)^2)+(15/8)*(ExtrS(j,size(A,2)+size(B,2)+size(C,2)+3)^4));
                AddLT3M(k,j)=((1-Wi(j,1))^2)*(Sigma(j,1)^2)*MSERegionSkew*(PKPS(j,1)^2);
                %Lambda(k,j)=GSep(i,1)+AddLT3M(k,j)+(((Sigma(j,1))^2)*(AddLT1M(k,j)+AddLT2M(k,j)+1+(ExtrS(j,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(j,size(A,2)+size(B,2)+size(C,2)+3))+(0.5*(ExtrS(j,size(A,2)+size(B,2)+size(C,2)+2))^2)*(1+0.75*ExtrS(j,size(A,2)+size(B,2)+size(C,2)+3)^2) ))/m(j,1);
                %---------------------
                %added by agv 8/6/15 for PRL fix, %PRL contains pseudo record lengths for each station for selected AEP.
                Lambda(k,j)=GSep(i,1)+AddLT3M(k,j)+(((Sigma(j,1))^2)*(AddLT1M(k,j)+AddLT2M(k,j)+1+(ExtrS(j,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(j,size(A,2)+size(B,2)+size(C,2)+3))+(0.5*(ExtrS(j,size(A,2)+size(B,2)+size(C,2)+2))^2)*(1+0.75*ExtrS(j,size(A,2)+size(B,2)+size(C,2)+3)^2) ))/PRL(j,1);
                %----------------------
            else
                AddLT1(k,j)=0.5*Wi(j,1)*ExtrS(k,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(j,size(A,2)+size(B,2)+size(C,2)+3)*PKPS(j,1)*(3*rij(k,j)+0.75*ExtrS(j,size(A,2)+size(B,2)+size(C,2)+3)*ExtrS(k,size(A,2)+size(B,2)+size(C,2)+3));
                AddLT2(k,j)=0.5*Wi(k,1)*ExtrS(j,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(k,size(A,2)+size(B,2)+size(C,2)+3)*PKPS(k,1)*(3*rij(k,j)+0.75*ExtrS(j,size(A,2)+size(B,2)+size(C,2)+3)*ExtrS(k,size(A,2)+size(B,2)+size(C,2)+3));
                AddLT3(k,j)=Wi(j,1)*Wi(k,1)*Sigma(j,1)*Sigma(k,1)*PKPS(j,1)*PKPS(k,1)*COVgg(j,k);
                %Lambda(k,j)=(rij(k,j)*Sigma(j,1)*Sigma(k,1)*Conm(k,j)*(AddLT1(k,j)+AddLT2(k,j)+AddLT3(k,j)+1+0.5*(ExtrS(j,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(j,size(A,2)+size(B,2)+size(C,2)+3))+0.5*(ExtrS(k,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(k,size(A,2)+size(B,2)+size(C,2)+3))+(0.5*ExtrS(j,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(k,size(A,2)+size(B,2)+size(C,2)+2))*(rij(k,j)+0.75*ExtrS(j,size(A,2)+size(B,2)+size(C,2)+3)*ExtrS(k,size(A,2)+size(B,2)+size(C,2)+3)) ) )/(m(j,1)*m(k,1));
                %---------------------
                %added by agv 8/6/15 for PRL fix, %PRL contains pseudo record lengths for each station for selected AEP.
                Lambda(k,j)=(rij(k,j)*Sigma(j,1)*Sigma(k,1)*Conm(k,j)*(AddLT1(k,j)+AddLT2(k,j)+AddLT3(k,j)+1+0.5*(ExtrS(j,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(j,size(A,2)+size(B,2)+size(C,2)+3))+0.5*(ExtrS(k,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(k,size(A,2)+size(B,2)+size(C,2)+3))+(0.5*ExtrS(j,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(k,size(A,2)+size(B,2)+size(C,2)+2))*(rij(k,j)+0.75*ExtrS(j,size(A,2)+size(B,2)+size(C,2)+3)*ExtrS(k,size(A,2)+size(B,2)+size(C,2)+3)) ) )/(PRL(j,1)*PRL(k,1));
                %---------------------
            end
        end
    end
    %GENERALIZED LEAST SQUARES REGRESSION USING LAMBDA WEIGHTING MATRIX
    XGLS=[ones(size(ExtrS,1),1) ExtrS(1:size(ExtrS,1), 1:size(A,2))];
    
    %added this segment to properly set up XGLS matrix for regression, AGV 12/14/2011
    Eq=isequal(XGLS(:,1),XGLS(:,2))
    if (Eq==1) && (size(XGLS,2)==2)
         XGLS = XGLS(:,1);
    end
    %End of section added by AGV  
    
    YGLS=[ExtrS(1:size(ExtrS,1), size(A,2)+ChY)];
    ZGLS=inv(XGLS'*inv(Lambda)*XGLS)*XGLS'*inv(Lambda)*YGLS;  
    GSepCoeff(:,i)=ZGLS;
    %Check whether GammaSq Estimate is Alright
    
    %changed Exp calc, changed for constant model by AGV, 12/14/2011
    if size(XGLS,2)==1
        Exp=size(ExtrS,1)-1; %for constant model
    else
        Exp=size(ExtrS,1)-size(A,2)-1;%n-p-1:n is the dimension of YHat and p is the dimension of coefficient matrix %For RoI, the model error variance will typically=0, so I will get ((YGLS-XGLS*ZGLS)')*ILambda*(YGLS-XGLS*ZGLS)<nmin-size(A,2)-1, so the sampling covariance matrix is explaining all the variability in the observed data. 
    end
    %end section changed by AGV
     
    Est=((YGLS-XGLS*ZGLS)')*inv(Lambda)*(YGLS-XGLS*ZGLS);
    GSQ(i,2)=Est-Exp;
end
%Refining the estimate of Gamma Squared (Model Error Variance)
for o=1:size(GSQ,1)
    if o==1 & GSQ(1,2)<0
        GSQrowmin1=1;
        break
    elseif GSQ(o,2)<0 & GSQ(o-1,2)>0 & o>1
        GSQRowEnd=o;
        GSQRowBeg=o-1;
        GSQrowmin1=-999;
        break    
    elseif o==size(GSQ,1)
        GSQrowmin1=30;
    else
        
    end
end
if GSQrowmin1==1 | GSQrowmin1==30
FinalCoeff(:,1)=GSepCoeff(:,GSQrowmin1);  %Selected GLS paramter estimates based on chosen gamma square value
SelGSQ=GSQ(GSQrowmin1,1); %Selected gamma square value (model error)

else
GInt2=30;
GSep2=linspace(GSQ(GSQRowBeg,1),GSQ(GSQRowEnd,1),GInt2)';
GSQ2=[GSep2(:,1) ones(size(GSep2,1),1)]
for p=1:size(GSep2,1)
    for q=1:size(ExtrS,1)
        for r=1:size(ExtrS,1)
            if q==r
                AddLT1M(r,q)=Wi(q,1)*ExtrS(q,size(A,2)+size(B,2)+size(C,2)+2)*PKPS(q,1)*(3*ExtrS(q,size(A,2)+size(B,2)+size(C,2)+3)+0.75*(ExtrS(q,size(A,2)+size(B,2)+size(C,2)+3))^(3));
                AddLT2M(r,q)=(Wi(q,1)^2)*(PKPS(q,1)^2)*(6+9*(ExtrS(q,size(A,2)+size(B,2)+size(C,2)+3)^2)+(15/8)*(ExtrS(q,size(A,2)+size(B,2)+size(C,2)+3)^4));
                AddLT3M(r,q)=((1-Wi(q,1))^2)*(Sigma(q,1)^2)*MSERegionSkew*(PKPS(q,1)^2);
                %Lambda2(r,q)=GSep2(p,1)+AddLT3M(r,q)+(((Sigma(q,1))^2)*(AddLT1M(r,q)+AddLT2M(r,q)+1+(ExtrS(q,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(q,size(A,2)+size(B,2)+size(C,2)+3))+(0.5*(ExtrS(q,size(A,2)+size(B,2)+size(C,2)+2))^2)*(1+0.75*ExtrS(q,size(A,2)+size(B,2)+size(C,2)+3)^2) ))/m(q,1);
                %---------------------
                %added by agv 8/6/15 for PRL fix, %PRL contains pseudo record lengths for each station for selected AEP.
                Lambda2(r,q)=GSep2(p,1)+AddLT3M(r,q)+(((Sigma(q,1))^2)*(AddLT1M(r,q)+AddLT2M(r,q)+1+(ExtrS(q,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(q,size(A,2)+size(B,2)+size(C,2)+3))+(0.5*(ExtrS(q,size(A,2)+size(B,2)+size(C,2)+2))^2)*(1+0.75*ExtrS(q,size(A,2)+size(B,2)+size(C,2)+3)^2) ))/PRL(q,1);
                %---------------------            
            else
                AddLT1(r,q)=0.5*Wi(q,1)*ExtrS(r,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(q,size(A,2)+size(B,2)+size(C,2)+3)*PKPS(q,1)*(3*rij(r,q)+0.75*ExtrS(q,size(A,2)+size(B,2)+size(C,2)+3)*ExtrS(r,size(A,2)+size(B,2)+size(C,2)+3));
                AddLT2(r,q)=0.5*Wi(r,1)*ExtrS(q,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(r,size(A,2)+size(B,2)+size(C,2)+3)*PKPS(r,1)*(3*rij(r,q)+0.75*ExtrS(q,size(A,2)+size(B,2)+size(C,2)+3)*ExtrS(r,size(A,2)+size(B,2)+size(C,2)+3));
                AddLT3(r,q)=Wi(q,1)*Wi(r,1)*Sigma(q,1)*Sigma(r,1)*PKPS(q,1)*PKPS(r,1)*COVgg(q,r);
                %Lambda2(r,q)=(rij(r,q)*Sigma(q,1)*Sigma(r,1)*Conm(r,q)*(AddLT1(r,q)+AddLT2(r,q)+AddLT3(r,q)+1+0.5*(ExtrS(q,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(q,size(A,2)+size(B,2)+size(C,2)+3))+0.5*(ExtrS(r,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(r,size(A,2)+size(B,2)+size(C,2)+3))+(0.5*ExtrS(q,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(r,size(A,2)+size(B,2)+size(C,2)+2))*(rij(r,q)+0.75*ExtrS(q,size(A,2)+size(B,2)+size(C,2)+3)*ExtrS(r,size(A,2)+size(B,2)+size(C,2)+3)) ) )/(m(q,1)*m(r,1));
                %---------------------
                %added by agv 8/6/15 for PRL fix, %PRL contains pseudo record lengths for each station for selected AEP.
                Lambda2(r,q)=(rij(r,q)*Sigma(q,1)*Sigma(r,1)*Conm(r,q)*(AddLT1(r,q)+AddLT2(r,q)+AddLT3(r,q)+1+0.5*(ExtrS(q,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(q,size(A,2)+size(B,2)+size(C,2)+3))+0.5*(ExtrS(r,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(r,size(A,2)+size(B,2)+size(C,2)+3))+(0.5*ExtrS(q,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(r,size(A,2)+size(B,2)+size(C,2)+2))*(rij(r,q)+0.75*ExtrS(q,size(A,2)+size(B,2)+size(C,2)+3)*ExtrS(r,size(A,2)+size(B,2)+size(C,2)+3)) ) )/(PRL(q,1)*PRL(r,1));
                %---------------------
            end
        end
    end
    %GENERALIZED LEAST SQUARES REGRESSION USING LAMBDA WEIGHTING MATRIX
    ZGLS2=inv(XGLS'*inv(Lambda2)*XGLS)*XGLS'*inv(Lambda2)*YGLS;  
    GSepCoeff2(:,p)=ZGLS2;
    %Check whether GammaSq Estimate is Alright
    
    %changed Exp calc, changed for constant model by AGV, 12/14/2011
    if size(XGLS,2)==1
        Exp2=size(ExtrS,1)-1; %for constant model
    else
        Exp2=size(ExtrS,1)-size(A,2)-1;%n-p-1:n is the dimension of YHat and p is the dimension of coefficient matrix %For RoI, the model error variance will typically=0, so I will get ((YGLS-XGLS*ZGLS)')*ILambda*(YGLS-XGLS*ZGLS)<nmin-size(A,2)-1, so the sampling covariance matrix is explaining all the variability in the observed data. 
    end
    %end section changed by AGV
    
    Est2=((YGLS-XGLS*ZGLS2)')*inv(Lambda2)*(YGLS-XGLS*ZGLS2);
    GSQ2(p,2)=Est2-Exp2;
end
for s=1:size(GSQ2,1)
    if s==1 & GSQ2(1,2)<0
        GSQrowmin=1;
    elseif GSQ2(s,2)<0 & GSQ2(s-1,2)>0 & s>1
        GSQrowmin=s-1;
        break    
    elseif s==size(GSQ2,1)
        GSQrowmin=30;
    else
        
    end
end
FinalCoeff(:,1)=GSepCoeff2(:,GSQrowmin);  %Selected GLS paramter estimates based on chosen gamma square value
SelGSQ=GSQ2(GSQrowmin,1); %Selected gamma square value (model error)
    
end

%Final Lambda matrix with selected gamma square value (model error)
for t=1:size(ExtrS,1)
    for u=1:size(ExtrS,1)
        if t==u
            AddLT1M(u,t)=Wi(t,1)*ExtrS(t,size(A,2)+size(B,2)+size(C,2)+2)*PKPS(t,1)*(3*ExtrS(t,size(A,2)+size(B,2)+size(C,2)+3)+0.75*(ExtrS(t,size(A,2)+size(B,2)+size(C,2)+3))^(3));
            AddLT2M(u,t)=(Wi(t,1)^2)*(PKPS(t,1)^2)*(6+9*(ExtrS(t,size(A,2)+size(B,2)+size(C,2)+3)^2)+(15/8)*(ExtrS(t,size(A,2)+size(B,2)+size(C,2)+3)^4));
            AddLT3M(u,t)=((1-Wi(t,1))^2)*(Sigma(t,1)^2)*MSERegionSkew*(PKPS(t,1)^2);
            %LambdaF(u,t)=SelGSQ+AddLT3M(u,t)+(((Sigma(t,1))^2)*(AddLT1M(u,t)+AddLT2M(u,t)+1+(ExtrS(t,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(t,size(A,2)+size(B,2)+size(C,2)+3))+(0.5*(ExtrS(t,size(A,2)+size(B,2)+size(C,2)+2))^2)*(1+0.75*ExtrS(t,size(A,2)+size(B,2)+size(C,2)+3)^2) ))/m(t,1);
            %---------------------
            %added by agv 8/6/15 for PRL fix, %PRL contains pseudo record lengths for each station for selected AEP.
            LambdaF(u,t)=SelGSQ+AddLT3M(u,t)+(((Sigma(t,1))^2)*(AddLT1M(u,t)+AddLT2M(u,t)+1+(ExtrS(t,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(t,size(A,2)+size(B,2)+size(C,2)+3))+(0.5*(ExtrS(t,size(A,2)+size(B,2)+size(C,2)+2))^2)*(1+0.75*ExtrS(t,size(A,2)+size(B,2)+size(C,2)+3)^2) ))/PRL(t,1);
            %---------------------
        else
            AddLT1(u,t)=0.5*Wi(t,1)*ExtrS(u,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(t,size(A,2)+size(B,2)+size(C,2)+3)*PKPS(t,1)*(3*rij(u,t)+0.75*ExtrS(t,size(A,2)+size(B,2)+size(C,2)+3)*ExtrS(u,size(A,2)+size(B,2)+size(C,2)+3));
            AddLT2(u,t)=0.5*Wi(u,1)*ExtrS(t,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(u,size(A,2)+size(B,2)+size(C,2)+3)*PKPS(u,1)*(3*rij(u,t)+0.75*ExtrS(t,size(A,2)+size(B,2)+size(C,2)+3)*ExtrS(u,size(A,2)+size(B,2)+size(C,2)+3));
            AddLT3(u,t)=Wi(t,1)*Wi(u,1)*Sigma(t,1)*Sigma(u,1)*PKPS(t,1)*PKPS(u,1)*COVgg(t,u);
            %LambdaF(u,t)=(rij(u,t)*Sigma(t,1)*Sigma(u,1)*Conm(u,t)*(AddLT1(u,t)+AddLT2(u,t)+AddLT3(u,t)+1+0.5*(ExtrS(t,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(t,size(A,2)+size(B,2)+size(C,2)+3))+0.5*(ExtrS(u,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(u,size(A,2)+size(B,2)+size(C,2)+3))+(0.5*ExtrS(t,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(u,size(A,2)+size(B,2)+size(C,2)+2))*(rij(u,t)+0.75*ExtrS(t,size(A,2)+size(B,2)+size(C,2)+3)*ExtrS(u,size(A,2)+size(B,2)+size(C,2)+3)) ) )/(m(t,1)*m(u,1));
            %---------------------
            %added by agv 8/6/15 for PRL fix, %PRL contains pseudo record lengths for each station for selected AEP.
            LambdaF(u,t)=(rij(u,t)*Sigma(t,1)*Sigma(u,1)*Conm(u,t)*(AddLT1(u,t)+AddLT2(u,t)+AddLT3(u,t)+1+0.5*(ExtrS(t,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(t,size(A,2)+size(B,2)+size(C,2)+3))+0.5*(ExtrS(u,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(u,size(A,2)+size(B,2)+size(C,2)+3))+(0.5*ExtrS(t,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(u,size(A,2)+size(B,2)+size(C,2)+2))*(rij(u,t)+0.75*ExtrS(t,size(A,2)+size(B,2)+size(C,2)+3)*ExtrS(u,size(A,2)+size(B,2)+size(C,2)+3)) ) )/(PRL(t,1)*PRL(u,1));
            %--------------------
        end
    end
end