function[FinalCoeff,SelGSQ,LambdaF]=LambdaMatrixFinalRoI(A,B,C,E,Sigma,ExtrS,Theta,Alpha,ChY,Conm);
%Generalized least squares (GLS) lambda weighting matrix for streamflow characteristics.

%'Sigma'-The sigma regression on each site's std dev of annual peak data vs basin attributes (performed on entire dataset) is in main program. The estimated std dev are in the 'Sigma' matrix.
%'ExtrS'-is a matrix holding the list of sites to use region-of-influence regression and associated basin attributes, std dev of annual peaks, POR, Lat, and Long.
%'A'-is the input matrix containing the predictors.
%'B'-is a matrix containing the predictands.
%'C'-is a matrix containing the Lat and Long.
%'E'-is a matrix containing the Lsigma, K, & G values.
%'F'-is a matrix containing the POR.
%'ChY'-is the column no. in the YHat data input matrix to use as the predictand.

%GEOGRAPHIC DISTANCE BETWEEN EACH STATION, 'dgeo' in the region-of-influence. Physical distance between each possible pair of stations in ROI
for a=1:size(ExtrS,1)
    for b=1:size(ExtrS,1)
        if a==b
            dgeo(b,a)=0;
        else
            dgeo(b,a)=((abs(ExtrS(a, size(A,2)+size(B,2)+1)-ExtrS(b, size(A,2)+size(B,2)+1))*1852*60)^(2)+(abs(ExtrS(a, size(A,2)+size(B,2)+2)-ExtrS(b, size(A,2)+size(B,2)+2))*1852*60)^(2))^(0.5);
        end
    end
end

%RECORD LENGTHS OF SITES IN ROI
for hh=1:size(ExtrS,1)
    m(hh,1)=ExtrS(hh,1+size(A,2)+size(B,2)+size(C,2)+size(E,2)+1+size(Sigma,2)+1);
end

%CONCURRENT RECORD LENGTHS FOR SITES IN THE ROI ONLY
for hingke=1:size(ExtrS,1)
    for hingkes=1:size(ExtrS,1)
        if hingke==hingkes
            Conm2(hingke,hingkes)=0;
        else
            Conm2(hingke,hingkes)=Conm(ExtrS(hingke,1+size(A,2)+size(B,2)+size(C,2)+size(E,2)+1+size(Sigma,2)),ExtrS(hingkes,1+size(A,2)+size(B,2)+size(C,2)+size(E,2)+1+size(Sigma,2)));        
        end
    end
end

%SAMPLE CROSS CORRELATION MATRIX, 'rij'. Empirical Cross Correlation Function Fit to All Stations that have a Specified No. Yrs of Concurrent Record (TO BE ADDED).
%Using the constants derived from another state (North Carolina, 50 Yrs of concurrent record based on ~1150 stations that had this)
for c=1:size(ExtrS,1)
    for d=1:size(ExtrS,1)
        if c==d
            rij(d,c)=1;
        else 
            rij(d,c)=Theta^(((dgeo(d,c)/1000)*0.6214)/((Alpha*((dgeo(d,c)/1000)*0.6214))+1));   %Emprirical relationship between geographic distances and correlations among annual peak series
        end
    end
end

%LAMBDA WEIGHTING MATRIX, Lambda. Assembling all the information in the previous subroutines to calculate Lambda
%An initial value of 'GammaSq' (ie model error) needs to be provided to start the iterative search for optimal b values. (Original method was the SECANT method from the Numerical Recipes book)
GammaSq1=0;
GammaSq2=2*(var(ExtrS(1:size(ExtrS,1),size(A,2)+ChY))); 
GInt=30;    
GSep=linspace(GammaSq1,GammaSq2,GInt)';
GSQ=[GSep(:,1) ones(size(GSep,1),2)];
for i=1:size(GSep,1)
    for j=1:size(ExtrS,1)
        for k=1:size(ExtrS,1)
            if j==k
                Lambda(k,j)=GSep(i,1)+(((Sigma(j,1))^2)*(1+(ExtrS(j,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(j,size(A,2)+size(B,2)+size(C,2)+3))+(0.5*(ExtrS(j,size(A,2)+size(B,2)+size(C,2)+2))^2)*(1+0.75*ExtrS(j,size(A,2)+size(B,2)+size(C,2)+3)^2) ))/m(j,1);
            else
                Lambda(k,j)=(rij(k,j)*Sigma(j,1)*Sigma(k,1)*Conm2(k,j)*(1+0.5*(ExtrS(j,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(j,size(A,2)+size(B,2)+size(C,2)+3))+0.5*(ExtrS(k,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(k,size(A,2)+size(B,2)+size(C,2)+3))+(0.5*ExtrS(j,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(k,size(A,2)+size(B,2)+size(C,2)+2))*(rij(k,j)+0.75*ExtrS(j,size(A,2)+size(B,2)+size(C,2)+3)*ExtrS(k,size(A,2)+size(B,2)+size(C,2)+3)) ) )/(m(j,1)*m(k,1));
            end
        end
    end
    %GENERALIZED LEAST SQUARES REGRESSION USING LAMBDA WEIGHTING MATRIX
    XGLS=[ones(size(ExtrS,1),1) ExtrS(1:size(ExtrS,1), 1:size(A,2))];
    YGLS=[ExtrS(1:size(ExtrS,1), size(A,2)+ChY)];
    ZGLS=inv(XGLS'*inv(Lambda)*XGLS)*XGLS'*inv(Lambda)*YGLS;  
    GSepCoeff(:,i)=ZGLS;
    %Check whether GammaSq Estimate is Alright
    Exp=size(ExtrS,1)-size(A,2)-1;    %n-p-1:n is the dimension of YHat and p is the dimension of coefficient matrix %For RoI, the model error variance will typically=0, so I will get ((YGLS-XGLS*ZGLS)')*ILambda*(YGLS-XGLS*ZGLS)<nmin-size(A,2)-1, so the sampling covariance matrix is explaining all the variability in the observed data. 
    Est=((YGLS-XGLS*ZGLS)')*inv(Lambda)*(YGLS-XGLS*ZGLS);
    GSQ(i,2)=Est-Exp;
end
%Refining the estimate of Gamma Squared (Model Error Variance)
for o=1:size(GSQ,1)
    if o==1 & GSQ(1,2)<0
        GSQrowmin1=1;
        break
    elseif GSQ(o,2)<0 & GSQ(o-1,2)>0 & o>1
        GSQRowEnd=o;
        GSQRowBeg=o-1;
        GSQrowmin1=-999;
        break    
    elseif o==size(GSQ,1)
        GSQrowmin1=30;
    else
        
    end
end
if GSQrowmin1==1 | GSQrowmin1==30
FinalCoeff(:,1)=GSepCoeff(:,GSQrowmin1);  %Selected GLS paramter estimates based on chosen gamma square value
SelGSQ=GSQ(GSQrowmin1,1); %Selected gamma square value (model error)

else
GInt2=30;
GSep2=linspace(GSQ(GSQRowBeg,1),GSQ(GSQRowEnd,1),GInt2)';
GSQ2=[GSep2(:,1) ones(size(GSep2,1),1)];
for p=1:size(GSep2,1)
    for q=1:size(ExtrS,1)
        for r=1:size(ExtrS,1)
            if q==r
                Lambda2(r,q)=GSep2(p,1)+(((Sigma(q,1))^2)*(1+(ExtrS(q,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(q,size(A,2)+size(B,2)+size(C,2)+3))+(0.5*(ExtrS(q,size(A,2)+size(B,2)+size(C,2)+2))^2)*(1+0.75*ExtrS(q,size(A,2)+size(B,2)+size(C,2)+3)^2) ))/m(q,1);
            else
                Lambda2(r,q)=(rij(r,q)*Sigma(q,1)*Sigma(r,1)*Conm2(r,q)*(1+0.5*(ExtrS(q,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(q,size(A,2)+size(B,2)+size(C,2)+3))+0.5*(ExtrS(r,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(r,size(A,2)+size(B,2)+size(C,2)+3))+(0.5*ExtrS(q,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(r,size(A,2)+size(B,2)+size(C,2)+2))*(rij(r,q)+0.75*ExtrS(q,size(A,2)+size(B,2)+size(C,2)+3)*ExtrS(r,size(A,2)+size(B,2)+size(C,2)+3)) ) )/(m(q,1)*m(r,1));
            end
        end
    end
    %GENERALIZED LEAST SQUARES REGRESSION USING LAMBDA WEIGHTING MATRIX
    ZGLS2=inv(XGLS'*inv(Lambda2)*XGLS)*XGLS'*inv(Lambda2)*YGLS;  
    GSepCoeff2(:,p)=ZGLS2;
    %Check whether GammaSq Estimate is Alright
    Exp2=size(ExtrS,1)-size(A,2)-1;    %n-p-1:n is the dimension of YHat and p is the dimension of coefficient matrix %For RoI, the model error variance will typically=0, so I will get ((YGLS-XGLS*ZGLS)')*ILambda*(YGLS-XGLS*ZGLS)<nmin-size(A,2)-1, so the sampling covariance matrix is explaining all the variability in the observed data. 
    Est2=((YGLS-XGLS*ZGLS2)')*inv(Lambda2)*(YGLS-XGLS*ZGLS2);
    GSQ2(p,2)=Est2-Exp2;
end
for s=1:size(GSQ2,1)
    if s==1 & GSQ2(1,2)<0
        GSQrowmin=1;
    elseif GSQ2(s,2)<0 & GSQ2(s-1,2)>0 & s>1
        GSQrowmin=s-1;
        break    
    elseif s==size(GSQ2,1)
        GSQrowmin=30;
    else
        
    end
end
FinalCoeff(:,1)=GSepCoeff2(:,GSQrowmin);  %Selected GLS paramter estimates based on chosen gamma square value
SelGSQ=GSQ2(GSQrowmin,1); %Selected gamma square value (model error)
    
end

%Final Lambda matrix with selected gamma square value (model error)
for t=1:size(ExtrS,1)
    for u=1:size(ExtrS,1)
        if t==u
            LambdaF(u,t)=SelGSQ+(((Sigma(t,1))^2)*(1+(ExtrS(t,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(t,size(A,2)+size(B,2)+size(C,2)+3))+(0.5*(ExtrS(t,size(A,2)+size(B,2)+size(C,2)+2))^2)*(1+0.75*ExtrS(t,size(A,2)+size(B,2)+size(C,2)+3)^2) ))/m(t,1);
        else
            LambdaF(u,t)=(rij(u,t)*Sigma(t,1)*Sigma(u,1)*Conm2(u,t)*(1+0.5*(ExtrS(t,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(t,size(A,2)+size(B,2)+size(C,2)+3))+0.5*(ExtrS(u,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(u,size(A,2)+size(B,2)+size(C,2)+3))+(0.5*ExtrS(t,size(A,2)+size(B,2)+size(C,2)+2)*ExtrS(u,size(A,2)+size(B,2)+size(C,2)+2))*(rij(u,t)+0.75*ExtrS(t,size(A,2)+size(B,2)+size(C,2)+3)*ExtrS(u,size(A,2)+size(B,2)+size(C,2)+3)) ) )/(m(t,1)*m(u,1));
        end
    end
end