function[Indsecterm,LevRoI,ProbPts2,NoLGLEVi,HLev,INFPts2,NoINFi,HLeverageLimit]=RoIMetrics(ExtrS,A,LambdaF,S,FinalCoeff,ChY);

%'ExtrS'-is a matrix holding the list of sites to use region-of-influence regression and associated basin attributes, std dev of annual peaks, POR, Lat, and Long.
%'A'-is the input matrix containing the predictors.
%'LambdaF'-is the final covariance matrix: Main diagonals=model error variance plus the variance at site, and Off diagonals=eqn 20 from Tasker and Stedinger (1989)-"An Operational GLS Model for Hydrologic Regression"
%'S'-is the sorted euclidean distance matrix from subroutine 'EucDistance'.
%'ChY'-is the column no. of the YHat to use as the predictand.

%Computing the second term values of the average variance of prediction (AVP) metric used for each GLS regression model
XAVP=[ones(size(ExtrS,1),1) ExtrS(1:size(ExtrS,1), 1:size(A,2))];
for a=1:size(XAVP,1)
    xavp=[1 ExtrS(a,1:size(A,2))];
    Indsecterm(a,1)=xavp*inv(XAVP'*inv(LambdaF)*XAVP)*xavp';
end

%Computing the RoI specific leverage statistic
%Compute the new GLS hat matrix
MultLev=4;  %Multiplier to Leverage Limit to apply (in conventional regression we use a value of 2).
%EstBC=[1 S(1, size(A,1)+1) S(1, size(A,1)+2) S(1, size(A,1)+3)];   %The basin characteristics of the site treated as ungaged and the RoI regression is built around it.
EstBC=[1 S(1, (size(A,1)+1):(size(A,1)+size(A,2)))];
XGLS=[ones(size(ExtrS,1),1) ExtrS(1:size(ExtrS,1), 1:size(A,2))];
YGLS=[ExtrS(1:size(ExtrS,1), size(A,2)+ChY)];
LevRoI=EstBC*inv(XGLS'*inv(LambdaF)*XGLS)*XGLS'*inv(LambdaF);  %Newly defined RoI hat vector of leverage values.
HLeverageLimit=(MultLev*sum(LevRoI(1,1:size(ExtrS,1))))/size(ExtrS,1);  %Any individual main diagonal component of the hat matrix exceeding this value has high leverage.
HLev=zeros(size(LevRoI,1),1);
NoLGLEVi=0;
for b=1:size(LevRoI,2)
    if abs(LevRoI(1,b))>HLeverageLimit
        HLev(b,1)=-999;    %Problem with high leverage point.    
        ProbPts2(1,b)=-999;
        NoLGLEVi=NoLGLEVi+1;
    else
        HLev(b,1)=1; %Point does not have high leverage.
        ProbPts2(1,b)=1;
    end       
end

%Computing the RoI specific influence statistic
Hgls=XGLS*inv(XGLS'*inv(LambdaF)*XGLS)*XGLS'*inv(LambdaF);
Hglsii=trace(Hgls); %equal to p
K=XGLS*inv(XGLS'*inv(LambdaF)*XGLS)*XGLS';
ei=YGLS-(XGLS*FinalCoeff(:,1));
NoINFi=0;
for d=1:size(LevRoI,2)   
    CooksD(1,d)=(1/Hglsii)*(abs(LevRoI(1,d)/Hgls(d,d)))*(((ei(d,1)^2)*K(d,d))/((LambdaF(d,d)-K(d,d))^2));
    if CooksD(1,d)>((MultLev*2)/size(ExtrS,1))
        INFPts2(1,d)=-999;
        NoINFi=NoINFi+1;
    else
        INFPts2(1,d)=1;
    end
end
            